local addonName, MaxDps = ...;

LibStub('AceAddon-3.0'):NewAddon(MaxDps, 'MaxDps', 'AceConsole-3.0', 'AceEvent-3.0', 'AceTimer-3.0');

--- @class MaxDps
_G[addonName] = MaxDps;

local TableInsert = tinsert;
local TableRemove = tremove;
local TableContains = tContains;
local TableIndexOf = tIndexOf;

local UnitIsFriend = UnitIsFriend;
local IsPlayerSpell = IsPlayerSpell;
local UnitClass = UnitClass;
local GetSpecialization = GetSpecialization;
local CreateFrame = CreateFrame;
local GetAddOnInfo = C_AddOns.GetAddOnInfo;
local IsAddOnLoaded = C_AddOns.IsAddOnLoaded
local LoadAddOn = C_AddOns.LoadAddOn;

local spellHistoryBlacklist = {
	[75] = true; -- Auto shot
};

function MaxDps:OnInitialize()
	self.db = LibStub('AceDB-3.0'):New('MaxDpsOptions', self.defaultOptions);

	self:RegisterChatCommand('maxdps', 'ShowMainWindow');

	if not self.db.global.customRotations then
		self.db.global.customRotations = {};
	end

	self:AddToBlizzardOptions();
end

function MaxDps:ShowMainWindow()
	if not self.Window then
		self.Window = self:GetModule('Window');
	end

	self.Window:ShowWindow();
end

function MaxDps:GetTexture()
	if self.db.global.customTexture ~= '' and self.db.global.customTexture ~= nil then
		self.FinalTexture = self.db.global.customTexture;
		return self.FinalTexture;
	end

	self.FinalTexture = self.db.global.texture;
	if self.FinalTexture == '' or self.FinalTexture == nil then
		self.FinalTexture = 'Interface\\Cooldown\\ping4';
	end

	return self.FinalTexture;
end

MaxDps.DefaultPrint = MaxDps.Print;
function MaxDps:Print(message,level)
	if not level then level = "info" end
	if self.db.global.disabledInfo and self.db.global.disabledInfo == "none" then
		return
	elseif self.db.global.disabledInfo and self.db.global.disabledInfo == "all" then
		MaxDps:DefaultPrint(message);
	elseif self.db.global.disabledInfo and self.db.global.disabledInfo == "errorinfo" then
		if level == "error" or level == "info" then
			MaxDps:DefaultPrint(message);
		end
	elseif self.db.global.disabledInfo and self.db.global.disabledInfo == "error" then
		if level == "error" then
			MaxDps:DefaultPrint(message);
		end
	elseif self.db.global.disabledInfo and self.db.global.disabledInfo == "info" then
		if level == "info" then
			MaxDps:DefaultPrint(message);
		end
	end
end

MaxDps.profilerStatus = 0;
function MaxDps:ProfilerStart()
	local profiler = self:GetModule('Profiler');
	profiler:StartProfiler();
	self.profilerStatus = 1;
end

function MaxDps:ProfilerStop()
	local profiler = self:GetModule('Profiler');
	profiler:StopProfiler();
	self.profilerStatus = 0;
end

function MaxDps:ProfilerToggle()
	if self.profilerStatus == 0 then
		self:ProfilerStart();
	else
		self:ProfilerStop();
	end
end

function MaxDps:EnableRotation()
	if self.NextSpell == nil or self.rotationEnabled then
		self:Print(self.Colors.Error .. 'Failed to enable addon!', "error");
		return
	end

	self:Fetch();
	self:UpdateButtonGlow();

	self:CheckTalents();
	self:GetAzeriteTraits();
	self:GetAzeriteEssences();
	self:GetCovenantInfo();
	self:GetLegendaryEffects();
	self:CheckIsPlayerMelee();
	if self.ModuleOnEnable then
		self.ModuleOnEnable();
	end

	self:EnableRotationTimer();

	self.rotationEnabled = true;
end

function MaxDps:EnableRotationTimer()
	self.RotationTimer = self:ScheduleRepeatingTimer('InvokeNextSpell', self.db.global.interval);
end

function MaxDps:DisableRotation()
	if not self.rotationEnabled then
		return
	end

	self:DisableRotationTimer();

	self:DestroyAllOverlays();
	self:Print(self.Colors.Info .. 'Disabling', "info");

	self.Spell = nil;
	self.rotationEnabled = false;
end

function MaxDps:DisableRotationTimer()
	if self.RotationTimer then
		self:CancelTimer(self.RotationTimer);
	end
end

local talentUpdateEvents = {
	"TRAIT_CONFIG_CREATED",
	"ACTIVE_COMBAT_CONFIG_CHANGED",
	"STARTER_BUILD_ACTIVATION_FAILED",
	"PLAYER_TALENT_UPDATE",
	"AZERITE_ESSENCE_ACTIVATED",
	"TRAIT_CONFIG_DELETED",
	"TRAIT_CONFIG_UPDATED",
	"LOADING_SCREEN_DISABLED",
}

function MaxDps:OnEnable()
	self:RegisterEvent('PLAYER_TARGET_CHANGED');
	self:RegisterEvent('PLAYER_REGEN_DISABLED');
	self:RegisterEvent('PLAYER_REGEN_ENABLED');

	for _, event in pairs(talentUpdateEvents) do
		self:RegisterEvent(event, 'TalentsUpdated')
	end

	self:RegisterEvent('ACTIONBAR_SLOT_CHANGED', 'ButtonFetch');
	self:RegisterEvent('ACTIONBAR_HIDEGRID', 'ButtonFetch');
	self:RegisterEvent('ACTIONBAR_PAGE_CHANGED', 'ButtonFetch');
	self:RegisterEvent('ACTIONBAR_UPDATE_STATE', 'ButtonFetch');
	self:RegisterEvent('UPDATE_BONUS_ACTIONBAR', 'ButtonFetch');
	self:RegisterEvent('LEARNED_SPELL_IN_TAB', 'ButtonFetch');
	self:RegisterEvent('CHARACTER_POINTS_CHANGED', 'ButtonFetch');
	self:RegisterEvent('ACTIVE_TALENT_GROUP_CHANGED', 'ButtonFetch');
	self:RegisterEvent('PLAYER_SPECIALIZATION_CHANGED', 'ButtonFetch');
	self:RegisterEvent('UPDATE_MACROS', 'ButtonFetch');
	self:RegisterEvent('VEHICLE_UPDATE', 'ButtonFetch');
	self:RegisterEvent('UPDATE_STEALTH', 'ButtonFetch');
	self:RegisterEvent('SPELLS_CHANGED', 'ButtonFetch');

	self:RegisterEvent('UNIT_ENTERED_VEHICLE');
	self:RegisterEvent('UNIT_EXITED_VEHICLE');

	self:RegisterEvent('NAME_PLATE_UNIT_ADDED');
	self:RegisterEvent('NAME_PLATE_UNIT_REMOVED');
	--	self:RegisterEvent('PLAYER_REGEN_ENABLED');

	if not self.playerUnitFrame then
		self.spellHistory = {};

		self.playerUnitFrame = CreateFrame('Frame');
		self.playerUnitFrame:RegisterUnitEvent('UNIT_SPELLCAST_SUCCEEDED', 'player');
		self.playerUnitFrame:SetScript('OnEvent', function(_, _, _, _, spellId)
			-- event, unit, lineId
			if IsPlayerSpell(spellId) and not spellHistoryBlacklist[spellId] then
				TableInsert(self.spellHistory, 1, spellId);

				if #self.spellHistory > 5 then
					TableRemove(self.spellHistory);
				end
			end
		end);
	end

	self:Print(self.Colors.Info .. 'Initialized', "info");
end

MaxDps.visibleNameplates = {};
function MaxDps:NAME_PLATE_UNIT_ADDED(_, nameplateUnit)
	if not TableContains(self.visibleNameplates, nameplateUnit) then
		TableInsert(self.visibleNameplates, nameplateUnit);
	end
end

function MaxDps:NAME_PLATE_UNIT_REMOVED(_, nameplateUnit)
	local index = TableIndexOf(self.visibleNameplates, nameplateUnit);
	if index ~= nil then
		TableRemove(self.visibleNameplates, index)
	end
end

function MaxDps:TalentsUpdated()
	self:DisableRotation();
	self:UpdateSpellsAndTalents()
end

local MaxDpsSpellTable
function MaxDps:UpdateSpellsAndTalents()
    local idtoclass = {
        [1] = "Warrior",
        [2] = "Paladin",
        [3] = "Hunter",
        [4] = "Rogue",
        [5] = "Priest",
        [6] = "Death Knight",
        [7] = "Shaman",
        [8] = "Mage",
        [9] = "Warlock",
        [10] = "Monk",
        [11] = "Druid",
        [12] = "Demon Hunter",
        [13] = "Evoker",
    }
	local idtospec = {
	    --Death Knight
	    [250] = "Blood",
	    [251] = "Frost",
	    [252] = "Unholy",
	    --Demon Hunter
	    [577] = "Havoc",
	    [581] = "Vengeance",
	    --Druid
	    [102] = "Balance",
	    [103] = "Feral",
	    [104] = "Guardian",
	    [105] = "Restoration",
		--Evoker
		[1473] = "Augmentation",
		[1467] = "Devastation",
		[1468] = "Preservation",
	    --Hunter
	    [253] = "Beast Mastery",
	    [254] = "Marksmanship",
	    [255] = "Survival",
	    --Mage
	    [62] = "Arcane",
	    [63] = "Fire",
	    [64] = "Frost",
	    --Monk
	    [268] = "Brewmaster",
	    [269] = "Windwalker",
	    [270] = "Mistweaver",
	    --Paladin
	    [65] = "Holy",
	    [66] = "Protection",
	    [70] = "Retribution",
	    --Priest
	    [256] = "Discipline",
	    [257] = "Holy",
	    [258] = "Shadow",
	    --Rogue
	    [259] = "Assassination",
	    [260] = "Outlaw",
	    [261] = "Subtlety",
	    --Shaman
	    [262] = "Elemental",
	    [263] = "Enhancement",
	    [264] = "Restoration",
	    --Warlock
	    [265] = "Affliction",
	    [266] = "Demonology",
	    [267] = "Destruction",
	    --Warrior
	    [71] = "Arms",
	    [72] = "Fury",
	    [73] = "Protection",
    }
	MaxDpsSpellTable = {
		["Warrior"] = {
		["Protection"] = {
		["SecondWind"] = 29838,
		["Charge"] = 100,
		["BatteringRam"] = 394312,
		["Uproar"] = 391572,
		["VictoryRush"] = 34428,
		["ArmoredtotheTeeth"] = 394855,
		["BraceForImpact"] = 386030,
		["Massacre"] = 281001,
		["DanceofDeath"] = 393965,
		["LeechingStrikes"] = 382258,
		["PiercingHowl"] = 12323,
		["DisruptingShout"] = 386071,
		["BoomingVoice"] = 202743,
		["Ravager"] = 228920,
		["ChallengingShout"] = 1161,
		["HeroicThrow"] = 57755,
		["OneHandedWeaponSpecialization"] = 382895,
		["Revenge"] = 6572,
		["Pummel"] = 6552,
		["HeavyRepercussions"] = 203177,
		["ThunderousWords"] = 384969,
		["FastFootwork"] = 382260,
		["SeismicReverberation"] = 382956,
		["ShieldWall"] = 871,
		["SonicBoom"] = 390725,
		["ShieldCharge"] = 385952,
		["SpellReflection"] = 23920,
		["BarbaricTraining"] = 390675,
		["Sidearm"] = 384404,
		["FocusedVigor"] = 384067,
		["DefendersAegis"] = 397103,
		["Hamstring"] = 1715,
		["Vanguard"] = 71,
		["SuddenDeath"] = 29725,
		["Taunt"] = 355,
		["WreckingThrow"] = 384110,
		["RumblingEarth"] = 275339,
		["LastStand"] = 12975,
		["CracklingThunder"] = 203201,
		["ThunderousRoar"] = 384318,
		["PainandGain"] = 382549,
		["FueledbyViolence"] = 383103,
		["Menace"] = 275338,
		["DeepWounds"] = 115768,
		["ImprovedHeroicThrow"] = 386034,
		["StormBolt"] = 107570,
		["Bloodsurge"] = 384361,
		["ShieldSlam"] = 23922,
		["PiercingChallenge"] = 382948,
		["MasteryCriticalBlock"] = 76857,
		["ChampionsMight"] = 386285,
		["ImpendingVictory"] = 202168,
		["Indomitable"] = 202095,
		["ShieldBlock"] = 2565,
		["EnduringAlacrity"] = 384063,
		["Shockwave"] = 46968,
		["Instigate"] = 394311,
		["Bloodborne"] = 385704,
		["DoubleTime"] = 103827,
		["Slam"] = 1464,
		["BestServedCold"] = 202560,
		["AngerManagement"] = 152278,
		["ShatteringThrow"] = 64382,
		["IntotheFray"] = 202603,
		["Bolster"] = 280001,
		["DefensiveStance"] = 386208,
		["Devastator"] = 236279,
		["ShieldSpecialization"] = 386011,
		["BerserkerShout"] = 384100,
		["Rend"] = 394062,
		["BattleShout"] = 6673,
		["Punish"] = 275334,
		["BitterImmunity"] = 383762,
		["IgnorePain"] = 190456,
		["BloodandThunder"] = 384277,
		["WarMachine"] = 316733,
		["HeroicLeap"] = 6544,
		["CruelStrikes"] = 392777,
		["BattleStance"] = 386164,
		["ConcussiveBlows"] = 383115,
		["EnduringDefenses"] = 386027,
		["FrothingBerserker"] = 392790,
		["HonedReflexes"] = 391271,
		["BerserkerRage"] = 18499,
		["BattleScarredVeteran"] = 386394,
		["CacophonousRoar"] = 382954,
		["CrushingForce"] = 390642,
		["BoundingStride"] = 202163,
		["OverwhelmingRage"] = 382767,
		["RallyingCry"] = 97462,
		["TitanicThrow"] = 384090,
		["ChampionsBulwark"] = 386328,
		["Juggernaut"] = 393967,
		["BrutalVitality"] = 384036,
		["DemoralizingShout"] = 1160,
		["Execute"] = 163201,
		["EnduranceTraining"] = 382940,
		["Strategist"] = 384041,
		["ImmovableObject"] = 394307,
		["ReinforcedPlates"] = 382939,
		["Avatar"] = 401150,
		["ShowofForce"] = 385843,
		["ToughasNails"] = 385888,
		["ThunderClap"] = 6343,
		["IntimidatingShout"] = 5246,
		["Thunderlord"] = 385840,
		["StormofSteel"] = 382953,
		["UnstoppableForce"] = 275336,
		["Whirlwind"] = 1680,
		["ChampionsSpear"] = 376079,
		["FuriousBlows"] = 390354,
		["Intervene"] = 3411,
		["WildStrikes"] = 382946,
		["ImpenetrableWall"] = 384072,
		["ViolentOutburst"] = 386477,
		["UnnervingFocus"] = 384042,
		["Devastate"] = 20243,
		["SpellBlock"] = 392966,
		["Riposte"] = 161798,
		},
		["Fury"] = {
		["SecondWind"] = 29838,
		["Charge"] = 100,
		["TitansTorment"] = 390135,
		["SwiftStrikes"] = 383459,
		["Uproar"] = 391572,
		["VictoryRush"] = 34428,
		["CriticalThinking"] = 383297,
		["ImprovedRagingBlow"] = 383854,
		["ArmoredtotheTeeth"] = 384124,
		["RagingBlow"] = 85288,
		["Ravager"] = 228920,
		["FreshMeat"] = 215568,
		["LeechingStrikes"] = 382258,
		["PiercingHowl"] = 12323,
		["Bloodcraze"] = 393950,
		["RagingArmaments"] = 388049,
		["Cruelty"] = 392931,
		["Pummel"] = 6552,
		["OdynsFury"] = 385059,
		["ThunderousWords"] = 384969,
		["DeftExperience"] = 383295,
		["SeismicReverberation"] = 382956,
		["Warpaint"] = 208154,
		["DualWieldSpecialization"] = 382900,
		["ImprovedEnrage"] = 383848,
		["SpellReflection"] = 23920,
		["Massacre"] = 206315,
		["Sidearm"] = 384404,
		["ThunderClap"] = 396719,
		["HeroicThrow"] = 57755,
		["SlaughteringStrikes"] = 388004,
		["Bloodthirst"] = 23881,
		["SuddenDeath"] = 280721,
		["Taunt"] = 355,
		["ChampionsSpear"] = 376079,
		["RumblingEarth"] = 275339,
		["EnragedRegeneration"] = 184364,
		["DoubleTime"] = 103827,
		["ThunderousRoar"] = 384318,
		["PainandGain"] = 382549,
		["Menace"] = 275338,
		["Shockwave"] = 46968,
		["FastFootwork"] = 382260,
		["SingleMindedFury"] = 81099,
		["DepthsofInsanity"] = 383922,
		["ImprovedExecute"] = 316402,
		["ShieldSlam"] = 23922,
		["ViciousContempt"] = 383885,
		["MasteryUnshackledFury"] = 76856,
		["ChampionsMight"] = 386285,
		["ImpendingVictory"] = 202168,
		["Rampage"] = 184367,
		["ShieldBlock"] = 2565,
		["IntimidatingShout"] = 5246,
		["Enrage"] = 184361,
		["WreckingThrow"] = 384110,
		["Bloodborne"] = 385703,
		["Avatar"] = 107574,
		["Slam"] = 1464,
		["CrushingForce"] = 382764,
		["AngerManagement"] = 152278,
		["ShatteringThrow"] = 64382,
		["WrathandFury"] = 392936,
		["Tenderize"] = 388933,
		["EnduranceTraining"] = 391997,
		["ReinforcedPlates"] = 382939,
		["CruelStrikes"] = 392777,
		["BerserkerShout"] = 384100,
		["TitanicThrow"] = 384090,
		["BattleShout"] = 6673,
		["TitanicRage"] = 394329,
		["BerserkerRage"] = 18499,
		["BerserkersTorment"] = 390123,
		["BloodandThunder"] = 384277,
		["Hurricane"] = 390563,
		["HeroicLeap"] = 6544,
		["DualWield"] = 231842,
		["StormofSwords"] = 388903,
		["TitansGrip"] = 46917,
		["FrothingBerserker"] = 215571,
		["FocusinChaos"] = 383486,
		["HonedReflexes"] = 391270,
		["RallyingCry"] = 97462,
		["Annihilator"] = 383916,
		["CacophonousRoar"] = 382954,
		["BitterImmunity"] = 383762,
		["BoundingStride"] = 202163,
		["OverwhelmingRage"] = 382767,
		["HackandSlash"] = 383877,
		["InvigoratingFury"] = 383468,
		["Execute"] = 163201,
		["Frenzy"] = 335077,
		["ImprovedWhirlwind"] = 12950,
		["WarMachine"] = 346002,
		["RecklessAbandon"] = 396749,
		["ConcussiveBlows"] = 383115,
		["StormBolt"] = 107570,
		["PiercingChallenge"] = 382948,
		["FrenziedFlurry"] = 383605,
		["DefensiveStance"] = 386208,
		["AshenJuggernaut"] = 392536,
		["CracklingThunder"] = 203201,
		["Recklessness"] = 1719,
		["Whirlwind"] = 1680,
		["Hamstring"] = 1715,
		["StormofSteel"] = 382953,
		["UnbridledFerocity"] = 389603,
		["ColdSteelHotBlood"] = 383959,
		["SonicBoom"] = 390725,
		["BarbaricTraining"] = 390674,
		["Intervene"] = 3411,
		["WildStrikes"] = 382946,
		["Onslaught"] = 315720,
		["FuriousBlows"] = 390354,
		["DancingBlades"] = 391683,
		["MeatCleaver"] = 280392,
		["ImprovedBloodthirst"] = 383852,
		["BerserkerStance"] = 386196,
		},
		["Arms"] = {
		["SecondWind"] = 29838,
		["BluntInstruments"] = 383442,
		["Skullsplitter"] = 260643,
		["VictoryRush"] = 34428,
		["CriticalThinking"] = 389306,
		["Massacre"] = 281001,
		["Overpower"] = 7384,
		["BlademastersTorment"] = 390138,
		["Charge"] = 100,
		["LeechingStrikes"] = 382258,
		["Warbreaker"] = 262161,
		["PiercingHowl"] = 12323,
		["DeftExperience"] = 389308,
		["MortalStrike"] = 12294,
		["MasteryDeepWounds"] = 262111,
		["InForTheKill"] = 248621,
		["ImprovedOverpower"] = 385571,
		["WildStrikes"] = 382946,
		["Pummel"] = 6552,
		["ChampionsMight"] = 386285,
		["ThunderousWords"] = 384969,
		["TwoHandedWeaponSpecialization"] = 382896,
		["SeismicReverberation"] = 382956,
		["Bloodsurge"] = 384361,
		["SonicBoom"] = 390725,
		["EnduranceTraining"] = 382940,
		["Unhinged"] = 386628,
		["BarbaricTraining"] = 383082,
		["Sidearm"] = 384404,
		["FastFootwork"] = 382260,
		["HeroicThrow"] = 57755,
		["FueledbyViolence"] = 383103,
		["Avatar"] = 107574,
		["FuriousBlows"] = 390354,
		["SweepingStrikes"] = 260708,
		["Impale"] = 383430,
		["Dreadnaught"] = 262150,
		["TideofBlood"] = 386357,
		["ShieldBlock"] = 2565,
		["SpellReflection"] = 23920,
		["PainandGain"] = 382549,
		["MercilessBonegrinder"] = 383317,
		["CrushingForce"] = 382764,
		["Battlelord"] = 386630,
		["MartialProwess"] = 316440,
		["FervorofBattle"] = 202316,
		["ImprovedExecute"] = 316405,
		["ShieldSlam"] = 23922,
		["PiercingChallenge"] = 382948,
		["Execute"] = 163201,
		["Tactician"] = 184783,
		["ImpendingVictory"] = 202168,
		["DefensiveStance"] = 386208,
		["CollateralDamage"] = 334779,
		["IntimidatingShout"] = 5246,
		["Shockwave"] = 46968,
		["WreckingThrow"] = 384110,
		["Bloodborne"] = 383287,
		["DoubleTime"] = 103827,
		["Slam"] = 1464,
		["Whirlwind"] = 1680,
		["AngerManagement"] = 152278,
		["ShatteringThrow"] = 64382,
		["IgnorePain"] = 190456,
		["BattleStance"] = 386164,
		["WarlordsTorment"] = 390140,
		["ColossusSmash"] = 167105,
		["Rend"] = 772,
		["BerserkerShout"] = 384100,
		["BitterImmunity"] = 383762,
		["BattleShout"] = 6673,
		["Fatality"] = 383703,
		["HonedReflexes"] = 382461,
		["ImprovedSweepingStrikes"] = 383155,
		["BloodandThunder"] = 384277,
		["Bladestorm"] = 227847,
		["HeroicLeap"] = 6544,
		["CruelStrikes"] = 392777,
		["StormofSwords"] = 385512,
		["ConcussiveBlows"] = 383115,
		["FrothingBerserker"] = 392792,
		["BoundingStride"] = 202163,
		["WarMachine"] = 262231,
		["RallyingCry"] = 97462,
		["ImprovedSlam"] = 400205,
		["CacophonousRoar"] = 382954,
		["RumblingEarth"] = 275339,
		["SpitefulSerenity"] = 400314,
		["OverwhelmingRage"] = 382767,
		["BerserkerRage"] = 18499,
		["Cleave"] = 845,
		["Hurricane"] = 390563,
		["Juggernaut"] = 383292,
		["ExhilaratingBlows"] = 383219,
		["SharpenedBlades"] = 383341,
		["DiebytheSword"] = 118038,
		["TestofMight"] = 385008,
		["StormBolt"] = 107570,
		["TitanicThrow"] = 384090,
		["ReinforcedPlates"] = 382939,
		["SeasonedSoldier"] = 279423,
		["Taunt"] = 355,
		["CracklingThunder"] = 203201,
		["StrengthofArms"] = 400803,
		["ReapingSwings"] = 383293,
		["Hamstring"] = 1715,
		["StormWall"] = 388807,
		["Menace"] = 275338,
		["ThunderousRoar"] = 384318,
		["ChampionsSpear"] = 376079,
		["SuddenDeath"] = 29725,
		["Intervene"] = 3411,
		["ExecutionersPrecision"] = 386634,
		["ThunderClap"] = 396719,
		["Bloodletting"] = 383154,
		["ArmoredtotheTeeth"] = 384124,
		["Uproar"] = 391572,
		["DanceofDeath"] = 390713,
		["ValorinVictory"] = 383338,
		},
		},
		["Paladin"] = {
		["Protection"] = {
		["ImprovedHolyShield"] = 393030,
		["HammerofJustice"] = 853,
		["TurnEvil"] = 10326,
		["Punishment"] = 403530,
		["Riposte"] = 161800,
		["BlessingofProtection"] = 1022,
		["SoaringShield"] = 378457,
		["HammerofWrath"] = 24275,
		["SeasonedWarhorse"] = 376996,
		["FaithsArmor"] = 406101,
		["DivineToll"] = 375576,
		["BlessedHammer"] = 204019,
		["SealofOrder"] = 385129,
		["HammeroftheRighteous"] = 53595,
		["BulwarkofRighteousFury"] = 386653,
		["Redoubt"] = 280373,
		["InmostLight"] = 405757,
		["ImprovedArdentDefender"] = 393114,
		["Cavalier"] = 230332,
		["CleanseToxins"] = 213644,
		["LayonHands"] = 633,
		["CrusaderStrike"] = 35395,
		["SealofMight"] = 385450,
		["UnbreakableSpirit"] = 114154,
		["ResoluteDefender"] = 385422,
		["FlashofLight"] = 19750,
		["GoldenPath"] = 377128,
		["SealofAlacrity"] = 385425,
		["AurasofSwiftVengeance"] = 385639,
		["Consecration"] = 26573,
		["AegisofLight"] = 353367,
		["Sanctuary"] = 379021,
		["Redemption"] = 7328,
		["HolyAegis"] = 385515,
		["InnerLight"] = 386568,
		["Rebuke"] = 96231,
		["DevotionAura"] = 465,
		["TirionsDevotion"] = 392928,
		["DivinePurpose"] = 223817,
		["AvengersShield"] = 31935,
		["CrusadersJudgment"] = 204023,
		["DivineResonance"] = 386738,
		["RelentlessInquisitor"] = 383388,
		["Justification"] = 377043,
		["JudgmentofLight"] = 183778,
		["FerrenMarcussFervor"] = 378762,
		["LightforgedBlessing"] = 406468,
		["GrandCrusader"] = 85043,
		["Contemplation"] = 121183,
		["HandoftheProtector"] = 315924,
		["QuickenedInvocation"] = 379391,
		["Judgment"] = 20271,
		["GreaterJudgment"] = 231663,
		["Obduracy"] = 385427,
		["MasteryDivineBulwark"] = 76671,
		["AurasoftheResolute"] = 385633,
		["UthersCounsel"] = 378425,
		["DivineShield"] = 642,
		["Intercession"] = 391054,
		["SealofReprisal"] = 377053,
		["TyrsEnforcer"] = 378285,
		["ArdentDefender"] = 31850,
		["StrengthinAdversity"] = 393071,
		["GuardianofAncientKings"] = 86659,
		["SacrificeoftheJust"] = 384820,
		["DivineSteed"] = 190784,
		["FocusedEnmity"] = 378845,
		["ZealotsParagon"] = 391142,
		["AvengingWrath"] = 31884,
		["FinalStand"] = 204077,
		["GiftoftheGoldenValkyr"] = 378279,
		["BastionofLight"] = 378974,
		["TouchofLight"] = 385349,
		["BulwarkofOrder"] = 209389,
		["SenseUndead"] = 5502,
		["MomentofGlory"] = 327193,
		["ImprovedBlessingofProtection"] = 384909,
		["BarricadeofFaith"] = 385726,
		["Recompense"] = 384914,
		["FadingLight"] = 405768,
		["EyeofTyr"] = 387174,
		["StrengthofConviction"] = 379008,
		["Incandescence"] = 385464,
		["SealofMercy"] = 384897,
		["SealoftheCrusader"] = 385728,
		["CrusadersReprieve"] = 403042,
		["SanctifiedPlates"] = 402964,
		["BlessingofFreedom"] = 1044,
		["ConsecrationinFlame"] = 379022,
		["BlindingLight"] = 115750,
		["LightoftheTitans"] = 378405,
		["SealofCharity"] = 384815,
		["OfDuskandDawn"] = 409441,
		["Sentinel"] = 389539,
		["ConsecratedGround"] = 204054,
		["ShiningLight"] = 321136,
		["BlessingofSacrifice"] = 6940,
		["FaithintheLight"] = 379043,
		["WordofGlory"] = 85673,
		["HolyShield"] = 152261,
		["UnboundFreedom"] = 305394,
		["Afterimage"] = 385414,
		["FistofJustice"] = 234299,
		["RighteousProtector"] = 204074,
		["ConcentrationAura"] = 317920,
		["BlessingofSpellwarding"] = 204018,
		["CrusadersResolve"] = 380188,
		["HandofReckoning"] = 62124,
		["InspiringVanguard"] = 393022,
		["ShieldoftheRighteous"] = 53600,
		["Repentance"] = 20066,
		},
		["Holy"] = {
		["Daybreak"] = 414170,
		["HammerofJustice"] = 853,
		["DivineGlimpse"] = 387805,
		["TurnEvil"] = 10326,
		["GloriousDawn"] = 414065,
		["Punishment"] = 403530,
		["HolyShock"] = 20473,
		["BlessingofProtection"] = 1022,
		["Cleanse"] = 4987,
		["ProtectionofTyr"] = 200430,
		["Veneration"] = 392938,
		["HammerofWrath"] = 24275,
		["HolyInfusion"] = 414214,
		["FaithsArmor"] = 406101,
		["DivineToll"] = 375576,
		["OverflowingLight"] = 414127,
		["DevotionAura"] = 465,
		["MasteryLightbringer"] = 183997,
		["TyrsDeliverance"] = 200652,
		["BlessingofFreedom"] = 1044,
		["LightofDawn"] = 85222,
		["Contemplation"] = 121183,
		["TowerofRadiance"] = 231642,
		["Absolution"] = 212056,
		["UnendingLight"] = 387998,
		["UntemperedDedication"] = 387814,
		["CommandingLight"] = 387781,
		["ShieldoftheRighteous"] = 53600,
		["CleanseToxins"] = 440013,
		["EmpyreanLegacy"] = 387170,
		["LayonHands"] = 633,
		["SanctifiedWrath"] = 53376,
		["CrusaderStrike"] = 35395,
		["Illumination"] = 414378,
		["UnbreakableSpirit"] = 114154,
		["HandofDivinity"] = 414273,
		["WordofGlory"] = 85673,
		["RighteousJudgment"] = 414113,
		["AuraMastery"] = 31821,
		["FlashofLight"] = 19750,
		["GoldenPath"] = 377128,
		["GlisteningRadiance"] = 414139,
		["LightsProtection"] = 416799,
		["AurasofSwiftVengeance"] = 385639,
		["Reclamation"] = 415364,
		["Awakening"] = 414195,
		["EchoingBlessings"] = 387801,
		["Awestruck"] = 417855,
		["BreakingDawn"] = 387879,
		["Redemption"] = 7328,
		["HolyAegis"] = 385515,
		["LightforgedBlessing"] = 406468,
		["BeaconoftheLightbringer"] = 197446,
		["InflorescenceoftheSunwell"] = 392907,
		["ConcentrationAura"] = 317920,
		["HandofReckoning"] = 62124,
		["QuickenedInvocation"] = 379391,
		["BarrierofFaith"] = 148039,
		["BlessedFocus"] = 414708,
		["RelentlessInquisitor"] = 383388,
		["RisingSunlight"] = 414203,
		["BlessingofAutumn"] = 388010,
		["ShiningRighteousness"] = 414443,
		["Repentance"] = 20066,
		["FadingLight"] = 405768,
		["JudgmentofLight"] = 183778,
		["StrengthofConviction"] = 379008,
		["DivineProtection"] = 498,
		["Judgment"] = 20271,
		["GreaterJudgment"] = 231644,
		["AurasoftheResolute"] = 385633,
		["FistofJustice"] = 234299,
		["HolyPrism"] = 114165,
		["Rebuke"] = 96231,
		["DivineShield"] = 642,
		["Intercession"] = 391054,
		["Obduracy"] = 385427,
		["Cavalier"] = 230332,
		["Incandescence"] = 385464,
		["CrusadersMight"] = 196926,
		["Recompense"] = 384914,
		["SacrificeoftheJust"] = 384820,
		["DivineSteed"] = 190784,
		["BeaconofFaith"] = 156910,
		["SealofMight"] = 385450,
		["AvengingWrath"] = 31884,
		["SealofMercy"] = 384897,
		["MomentofCompassion"] = 387786,
		["ImprovedBlessingofProtection"] = 384909,
		["TouchofLight"] = 385349,
		["DivineRevelations"] = 387808,
		["SenseUndead"] = 5502,
		["HolyLight"] = 82326,
		["ResplendentLight"] = 392902,
		["BeaconofLight"] = 53563,
		["SealofOrder"] = 385129,
		["UnwaveringSpirit"] = 392911,
		["SealofAlacrity"] = 385425,
		["LightsHammer"] = 114158,
		["SeasonedWarhorse"] = 376996,
		["GlimmerofLight"] = 287269,
		["SealoftheCrusader"] = 416770,
		["CrusadersReprieve"] = 403042,
		["VanguardsMomentum"] = 416869,
		["ImprovedCleanse"] = 393024,
		["SanctifiedPlates"] = 402964,
		["BlindingLight"] = 115750,
		["LightoftheMartyr"] = 183998,
		["DivineResonance"] = 386738,
		["OfDuskandDawn"] = 409439,
		["SavedbytheLight"] = 157047,
		["AvengingCrusader"] = 216331,
		["ImbuedInfusions"] = 392961,
		["BlessingofSacrifice"] = 6940,
		["PoweroftheSilverHand"] = 200474,
		["InfusionofLight"] = 53576,
		["DivinePurpose"] = 223817,
		["LightsConviction"] = 414073,
		["Afterimage"] = 385414,
		["DivineFavor"] = 210294,
		["Consecration"] = 26573,
		["BoundlessSalvation"] = 392951,
		["TirionsDevotion"] = 414720,
		["MaraadsDyingBreath"] = 388018,
		["BeaconofVirtue"] = 200025,
		["MercifulAuras"] = 183415,
		["Justification"] = 377043,
		["BlessingofSummer"] = 388007,
		},
		["Retribution"] = {
		["BlessedChampion"] = 403010,
		["HeartoftheCrusader"] = 406154,
		["FinalVerdict"] = 383328,
		["Punishment"] = 403530,
		["Penitence"] = 403026,
		["BlessingofProtection"] = 1022,
		["TempestoftheLightbringer"] = 383396,
		["Incandescence"] = 385464,
		["HammerofWrath"] = 24275,
		["SeasonedWarhorse"] = 376996,
		["FaithsArmor"] = 406101,
		["DivineToll"] = 375576,
		["Cavalier"] = 230332,
		["Obduracy"] = 385427,
		["Justification"] = 377043,
		["DivineAuxiliary"] = 406158,
		["BlessingofFreedom"] = 1044,
		["ZealotsFervor"] = 403509,
		["BladeofJustice"] = 184575,
		["ArtofWar"] = 406064,
		["ImprovedJudgment"] = 405461,
		["DivineHammer"] = 198034,
		["HolyAegis"] = 385515,
		["SwiftJustice"] = 383228,
		["ShieldoftheRighteous"] = 53600,
		["CleanseToxins"] = 213644,
		["ExecutionSentence"] = 343527,
		["LayonHands"] = 633,
		["LightofJustice"] = 404436,
		["CrusaderStrike"] = 35395,
		["SealofMight"] = 385450,
		["BurningCrusade"] = 405289,
		["TruthsWake"] = 403696,
		["JusticarsVengeance"] = 215661,
		["DivinePurpose"] = 408459,
		["TemplarsVerdict"] = 85256,
		["FlashofLight"] = 19750,
		["GoldenPath"] = 377128,
		["SealofAlacrity"] = 385425,
		["DivineArbiter"] = 404306,
		["AurasofSwiftVengeance"] = 385639,
		["Consecration"] = 26573,
		["UnbreakableSpirit"] = 114154,
		["TemplarStrikes"] = 406646,
		["UnboundFreedom"] = 305394,
		["ImprovedCrusaderStrike"] = 383254,
		["HighlordsJudgment"] = 404512,
		["Crusade"] = 231895,
		["BlessingofSacrifice"] = 6940,
		["Rebuke"] = 96231,
		["SealofOrder"] = 385129,
		["Jurisdiction"] = 402971,
		["HandofReckoning"] = 62124,
		["BladesofLight"] = 403664,
		["ConsecratedGround"] = 204054,
		["DivineResonance"] = 384027,
		["ConsecratedBlade"] = 404834,
		["DivineStorm"] = 53385,
		["RighteousCause"] = 402912,
		["LightsCelerity"] = 403698,
		["GuidedPrayer"] = 404357,
		["FadingLight"] = 405768,
		["LightforgedBlessing"] = 403479,
		["ImprovedBlessingofProtection"] = 384909,
		["DivineProtection"] = 403876,
		["Judgment"] = 20271,
		["GreaterJudgment"] = 231663,
		["CrusadingStrikes"] = 404542,
		["InstrumentofRetribution"] = 404752,
		["AurasoftheResolute"] = 385633,
		["Adjudication"] = 406157,
		["JudgmentofLight"] = 183778,
		["Intercession"] = 391054,
		["HealingHands"] = 326734,
		["Sanctify"] = 382536,
		["SearingLight"] = 404540,
		["SealoftheCrusader"] = 385728,
		["Recompense"] = 384914,
		["SacrificeoftheJust"] = 384820,
		["DivineSteed"] = 190784,
		["FinalReckoning"] = 343721,
		["RushofLight"] = 407067,
		["AvengingWrath"] = 31884,
		["SealofMercy"] = 384897,
		["CrusadersReprieve"] = 403042,
		["MasteryHandofLight"] = 267316,
		["TouchofLight"] = 385349,
		["StrengthofConviction"] = 379008,
		["SenseUndead"] = 5502,
		["EmpyreanPower"] = 326732,
		["BladeofVengeance"] = 403826,
		["ImprovedBladeofJustice"] = 403745,
		["VanguardsMomentum"] = 383314,
		["VanguardofJustice"] = 406545,
		["Expurgation"] = 383344,
		["HolyBlade"] = 383342,
		["RetributionAura"] = 183435,
		["AegisofProtection"] = 403654,
		["InquisitorsIre"] = 403975,
		["EmpyreanLegacy"] = 387170,
		["SanctifiedPlates"] = 402964,
		["ShieldofVengeance"] = 184662,
		["QuickenedInvocation"] = 379391,
		["BlindingLight"] = 115750,
		["ExecutionersWill"] = 406940,
		["TurnEvil"] = 10326,
		["OfDuskandDawn"] = 385125,
		["SeethingFlames"] = 405355,
		["WakeofAshes"] = 255937,
		["FistofJustice"] = 234299,
		["JudgeJuryandExecutioner"] = 405607,
		["DevotionAura"] = 465,
		["WordofGlory"] = 85673,
		["JudgmentofJustice"] = 403495,
		["ConcentrationAura"] = 317920,
		["Afterimage"] = 385414,
		["Contemplation"] = 121183,
		["BoundlessJudgment"] = 405278,
		["DivineShield"] = 642,
		["HammerofJustice"] = 853,
		["CrusaderAura"] = 32223,
		["DivineWrath"] = 406872,
		["Redemption"] = 7328,
		["VengefulWrath"] = 406835,
		["Repentance"] = 20066,
		},
		},
		["Shaman"] = {
		["Enhancement"] = {
		["ForcefulWinds"] = 262647,
		["Stormflurry"] = 344357,
		["NaturesGuardian"] = 30884,
		["FlameShock"] = 188389,
		["PlanesTraveler"] = 381647,
		["Stormblast"] = 319930,
		["StaticAccumulation"] = 384411,
		["Flurry"] = 382888,
		["WindfuryWeapon"] = 33757,
		["WitchDoctorsAncestry"] = 384447,
		["FeralSpirit"] = 51533,
		["EarthbindTotem"] = 2484,
		["CrashingStorms"] = 334308,
		["GowiththeFlow"] = 381678,
		["FrostShock"] = 196840,
		["ElementalOrbit"] = 383010,
		["ElementalWeapons"] = 384355,
		["FireNova"] = 333974,
		["VoodooMastery"] = 204268,
		["AstralRecall"] = 556,
		["FeralLunge"] = 196884,
		["PrimalStrike"] = 73899,
		["AlphaWolf"] = 198434,
		["HealingSurge"] = 8004,
		["ThorimsInvocation"] = 384444,
		["GuardiansCudgel"] = 381819,
		["FlametongueWeapon"] = 318038,
		["RagingMaelstrom"] = 384143,
		["CalloftheElements"] = 383011,
		["StoneskinTotem"] = 383017,
		["SpiritWalk"] = 58875,
		["CrashLightning"] = 187874,
		["StaticCharge"] = 265046,
		["ElementalWarding"] = 381650,
		["ConvergingStorms"] = 384363,
		["TempestStrikes"] = 428071,
		["ChainLightning"] = 188443,
		["NaturesFury"] = 381655,
		["Hex"] = 51514,
		["TotemicProjection"] = 108287,
		["TranquilAirTotem"] = 383019,
		["DeeplyRootedElements"] = 378270,
		["Bloodlust"] = 2825,
		["LavaLash"] = 60103,
		["AncestralWolfAffinity"] = 382197,
		["ThunderousPaws"] = 378075,
		["SpiritWolf"] = 260878,
		["DoomWinds"] = 384352,
		["LegacyoftheFrostWitch"] = 384450,
		["Enfeeblement"] = 378079,
		["TotemicRecall"] = 108285,
		["TotemicSurge"] = 381867,
		["FarSight"] = 6196,
		["AstralShift"] = 108271,
		["WindsofAlAkir"] = 382215,
		["Thundershock"] = 378779,
		["ManaSpring"] = 381930,
		["AncestralGuidance"] = 108281,
		["SwirlingMaelstrom"] = 384359,
		["AncestralDefense"] = 382947,
		["HealingStreamTotem"] = 5394,
		["ElementalBlast"] = 117014,
		["LightningLasso"] = 305483,
		["Stormstrike"] = 17364,
		["CapacitorTotem"] = 192058,
		["Thunderstorm"] = 51490,
		["EarthElemental"] = 198103,
		["AncestralSpirit"] = 2008,
		["GreaterPurge"] = 378773,
		["SpiritwalkersAegis"] = 378077,
		["SplinteredElements"] = 382042,
		["LightningShield"] = 192106,
		["MasteryEnhancedElements"] = 77223,
		["CleanseSpirit"] = 51886,
		["ElementalSpirits"] = 262624,
		["AstralBulwark"] = 377933,
		["SpiritwalkersGrace"] = 79206,
		["SwirlingCurrents"] = 378094,
		["FocusedInsight"] = 381666,
		["OverflowingMaelstrom"] = 384149,
		["RefreshingWaters"] = 393905,
		["ElementalAssault"] = 210853,
		["Sundering"] = 197214,
		["GracefulSpirit"] = 192088,
		["WaterWalking"] = 546,
		["CreationCore"] = 383012,
		["GustofWind"] = 192063,
		["BrimmingwithLife"] = 381689,
		["WindRushTotem"] = 192077,
		["EarthgrabTotem"] = 51485,
		["TremorTotem"] = 8143,
		["Hailstorm"] = 334195,
		["GhostWolf"] = 2645,
		["AshenCatalyst"] = 390370,
		["NaturesSwiftness"] = 378081,
		["UnrulyWinds"] = 390288,
		["ChainHeal"] = 1064,
		["LashingFlames"] = 334046,
		["FireandIce"] = 382886,
		["WindfuryTotem"] = 8512,
		["WindShear"] = 57994,
		["PoisonCleansingTotem"] = 383013,
		["Reincarnation"] = 20608,
		["HotHand"] = 201900,
		["MaelstromWeapon"] = 187880,
		["Purge"] = 370,
		["TotemicFocus"] = 382201,
		["MoltenAssault"] = 334033,
		["LightningBolt"] = 188196,
		["SurgingShields"] = 382033,
		["LavaBurst"] = 51505,
		["IceStrike"] = 342240,
		["PrimordialWave"] = 375982,
		["PrimalMaelstrom"] = 384405,
		["ImprovedMaelstromWeapon"] = 383303,
		["Ascendance"] = 114051,
		["EarthShield"] = 974,
		["StormsWrath"] = 392352,
		},
		["Restoration"] = {
		["ElementalOrbit"] = 383010,
		["Reincarnation"] = 20608,
		["TidalWaves"] = 51564,
		["CurrentControl"] = 404015,
		["EarthShield"] = 974,
		["Downpour"] = 207778,
		["NaturesGuardian"] = 30884,
		["FlameShock"] = 188389,
		["PlanesTraveler"] = 381647,
		["AstralRecall"] = 556,
		["Wellspring"] = 197995,
		["Flurry"] = 382888,
		["MasteroftheElements"] = 16166,
		["LivingStream"] = 382482,
		["LavaSurge"] = 77756,
		["EarthbindTotem"] = 2484,
		["PurifySpirit"] = 77130,
		["Resurgence"] = 16196,
		["AcidRain"] = 378443,
		["GowiththeFlow"] = 381678,
		["VoodooMastery"] = 204268,
		["SurgingShields"] = 382033,
		["Undercurrent"] = 382194,
		["FrostShock"] = 196840,
		["TotemicSurge"] = 381867,
		["CloudburstTotem"] = 157153,
		["AncestralProtectionTotem"] = 207399,
		["Bloodlust"] = 2825,
		["FlashFlood"] = 280614,
		["TumblingWaves"] = 382040,
		["PrimalStrike"] = 73899,
		["WindShear"] = 57994,
		["ThunderousPaws"] = 378075,
		["TotemicProjection"] = 108287,
		["StoneskinTotem"] = 383017,
		["FlametongueWeapon"] = 318038,
		["AncestralVision"] = 212048,
		["CalloftheElements"] = 383011,
		["HealingWave"] = 77472,
		["SpiritWalk"] = 58875,
		["Deluge"] = 200076,
		["UnleashLife"] = 73685,
		["StaticCharge"] = 265046,
		["ImprovedPrimordialWave"] = 382191,
		["Riptide"] = 61295,
		["ChainLightning"] = 188443,
		["HighTide"] = 157154,
		["DeeplyRootedElements"] = 378270,
		["TotemicFocus"] = 382201,
		["TranquilAirTotem"] = 383019,
		["MasteryDeepHealing"] = 77226,
		["Enfeeblement"] = 378079,
		["LavaBurst"] = 51505,
		["AncestralWolfAffinity"] = 382197,
		["OverflowingShores"] = 383222,
		["SpiritWolf"] = 260878,
		["EarthgrabTotem"] = 51485,
		["AncestralGuidance"] = 108281,
		["EarthenWallTotem"] = 198838,
		["WaterShield"] = 52127,
		["ElementalWarding"] = 381650,
		["FarSight"] = 6196,
		["AstralShift"] = 108271,
		["WindsofAlAkir"] = 382215,
		["Thundershock"] = 378779,
		["ContinuousWaves"] = 382046,
		["ChainHeal"] = 1064,
		["MaelstromWeapon"] = 187880,
		["Undulation"] = 200071,
		["EarthlivingWeapon"] = 382021,
		["SpiritwalkersTidalTotem"] = 404522,
		["WindRushTotem"] = 192077,
		["SpiritLinkTotem"] = 98008,
		["CapacitorTotem"] = 192058,
		["Thunderstorm"] = 51490,
		["EarthElemental"] = 198103,
		["CleanseSpirit"] = 440012,
		["TideTurner"] = 404019,
		["WaterWalking"] = 546,
		["GreaterPurge"] = 378773,
		["LightningShield"] = 192106,
		["ImprovedEarthlivingWeapon"] = 382315,
		["WaterTotemMastery"] = 382030,
		["AncestralAwakening"] = 382309,
		["AstralBulwark"] = 377933,
		["SpiritwalkersGrace"] = 79206,
		["Stormkeeper"] = 383009,
		["Torrent"] = 200072,
		["ResonantWaters"] = 404539,
		["RefreshingWaters"] = 378211,
		["HealingTideTotem"] = 108280,
		["AncestralReach"] = 382732,
		["GracefulSpirit"] = 192088,
		["GustofWind"] = 192063,
		["PrimalTideCore"] = 382045,
		["NaturesFury"] = 381655,
		["BrimmingwithLife"] = 381689,
		["WavespeakersBlessing"] = 381946,
		["CreationCore"] = 383012,
		["TremorTotem"] = 8143,
		["FlowoftheTides"] = 382039,
		["GhostWolf"] = 2645,
		["SwirlingCurrents"] = 378094,
		["NaturesSwiftness"] = 378081,
		["SpiritwalkersAegis"] = 378077,
		["AncestralVigor"] = 207401,
		["ImprovedPurifySpirit"] = 383016,
		["FireandIce"] = 382886,
		["AncestralSpirit"] = 2008,
		["PrimordialWave"] = 428332,
		["HealingRain"] = 73920,
		["PoisonCleansingTotem"] = 383013,
		["AncestralDefense"] = 382947,
		["Hex"] = 51514,
		["EchooftheElements"] = 333919,
		["Purge"] = 370,
		["TotemicRecall"] = 108285,
		["LightningBolt"] = 188196,
		["Tidebringer"] = 236501,
		["HealingStreamTotem"] = 5394,
		["LightningLasso"] = 305483,
		["ManaSpring"] = 381930,
		["GuardiansCudgel"] = 381819,
		["HealingSurge"] = 8004,
		["Ascendance"] = 114052,
		["EarthenHarmony"] = 382020,
		["ManaTideTotem"] = 16191,
		},
		["Elemental"] = {
		["UnrelentingCalamity"] = 382685,
		["MountainsWillFall"] = 381726,
		["NaturesGuardian"] = 30884,
		["Reincarnation"] = 20608,
		["FlamesoftheCauldron"] = 378266,
		["PlanesTraveler"] = 381647,
		["CreationCore"] = 383012,
		["LavaSurge"] = 77756,
		["Flurry"] = 382888,
		["MasteroftheElements"] = 16166,
		["AstralRecall"] = 556,
		["ImprovedFlametongueWeapon"] = 382027,
		["Icefury"] = 210714,
		["SurgeofPower"] = 262303,
		["LightningLasso"] = 305483,
		["LiquidMagmaTotem"] = 192222,
		["GowiththeFlow"] = 381678,
		["FlowofPower"] = 385923,
		["PrimalElementalist"] = 117013,
		["FlameShock"] = 188389,
		["FrostShock"] = 196840,
		["SurgingShields"] = 382033,
		["Aftershock"] = 273221,
		["Bloodlust"] = 2825,
		["PrimalStrike"] = 73899,
		["TotemicRecall"] = 108285,
		["PoweroftheMaelstrom"] = 191861,
		["VoodooMastery"] = 204268,
		["Inundate"] = 378776,
		["ManaSpring"] = 381930,
		["CalloftheElements"] = 383011,
		["StoneskinTotem"] = 383017,
		["FlametongueWeapon"] = 318038,
		["OathoftheFarSeer"] = 381785,
		["MasteryElementalOverload"] = 168534,
		["TotemicSurge"] = 381867,
		["SpiritWalk"] = 58875,
		["CallofFire"] = 378255,
		["Earthquake"] = 61882,
		["StaticCharge"] = 265046,
		["ElectrifiedShocks"] = 382086,
		["ThunderousPaws"] = 378075,
		["ChainLightning"] = 188443,
		["NaturesFury"] = 381655,
		["DeeplyRootedElements"] = 378270,
		["TotemicProjection"] = 108287,
		["TranquilAirTotem"] = 383019,
		["StormElemental"] = 192249,
		["ElementalEquilibrium"] = 378271,
		["LavaBurst"] = 51505,
		["AncestralWolfAffinity"] = 382197,
		["GustofWind"] = 192063,
		["SpiritWolf"] = 260878,
		["EchooftheElements"] = 333919,
		["AncestralGuidance"] = 108281,
		["PrimordialFury"] = 378193,
		["PrimordialBond"] = 381764,
		["ElementalFury"] = 60188,
		["FarSight"] = 6196,
		["AstralShift"] = 108271,
		["MagmaChamber"] = 381932,
		["Thundershock"] = 378779,
		["PoisonCleansingTotem"] = 383013,
		["Purge"] = 370,
		["MaelstromWeapon"] = 187880,
		["LightningRod"] = 210689,
		["CallofThunder"] = 378241,
		["ElementalBlast"] = 117014,
		["SplinteredElements"] = 382042,
		["FurtherBeyond"] = 381787,
		["CapacitorTotem"] = 192058,
		["Thunderstorm"] = 51490,
		["EarthElemental"] = 198103,
		["WaterWalking"] = 546,
		["EyeoftheStorm"] = 381708,
		["EarthShock"] = 8042,
		["GreaterPurge"] = 378773,
		["LightningShield"] = 192106,
		["TumultuousFissures"] = 381743,
		["CleanseSpirit"] = 51886,
		["SpiritwalkersGrace"] = 79206,
		["AstralBulwark"] = 377933,
		["WindspeakersLavaResurgence"] = 378268,
		["BrimmingwithLife"] = 381689,
		["FocusedInsight"] = 381666,
		["Stormkeeper"] = 191634,
		["RefreshingWaters"] = 378211,
		["ElementalWarding"] = 381650,
		["RollingMagma"] = 386443,
		["GracefulSpirit"] = 192088,
		["WindRushTotem"] = 192077,
		["EarthbindTotem"] = 2484,
		["EarthgrabTotem"] = 51485,
		["Hex"] = 51514,
		["Enfeeblement"] = 378079,
		["EchoChamber"] = 382032,
		["TremorTotem"] = 8143,
		["SpiritwalkersAegis"] = 378077,
		["GhostWolf"] = 2645,
		["SwirlingCurrents"] = 378094,
		["NaturesSwiftness"] = 378081,
		["WindsofAlAkir"] = 382215,
		["ChainHeal"] = 1064,
		["SkybreakersFieryDemise"] = 378310,
		["FireandIce"] = 382886,
		["AncestralSpirit"] = 2008,
		["WindShear"] = 57994,
		["EchoesofGreatSundering"] = 384087,
		["TotemicFocus"] = 382201,
		["AncestralDefense"] = 382947,
		["PrimordialSurge"] = 386474,
		["SearingFlames"] = 381782,
		["FlashofLightning"] = 381936,
		["FireElemental"] = 198067,
		["LightningBolt"] = 188196,
		["HealingStreamTotem"] = 5394,
		["FluxMelting"] = 381776,
		["EarthShield"] = 974,
		["PrimordialWave"] = 375982,
		["GuardiansCudgel"] = 381819,
		["SwellingMaelstrom"] = 381707,
		["Ascendance"] = 114050,
		["HealingSurge"] = 8004,
		["ElementalOrbit"] = 383010,
		},
		},
		["Monk"] = {
		["Windwalker"] = {
		["CalmingPresence"] = 388664,
		["GenerousPour"] = 389575,
		["MarkoftheCrane"] = 220357,
		["Skyreach"] = 392991,
		["FortifyingBrew"] = 115203,
		["JadefireHarmony"] = 391412,
		["Ascension"] = 115396,
		["LastEmperorsCapacitor"] = 392989,
		["Transcendence"] = 101643,
		["DrinkingHornCover"] = 391370,
		["JadefireStomp"] = 388193,
		["XuensBond"] = 392986,
		["Serenity"] = 152173,
		["InvokersDelight"] = 388661,
		["StrengthofSpirit"] = 387276,
		["ProfoundRebuttal"] = 392910,
		["TouchoftheTiger"] = 388856,
		["DanceoftheWind"] = 432181,
		["ZenFlight"] = 125883,
		["ZenPilgrimage"] = 126892,
		["GraceoftheCrane"] = 388811,
		["BounceBack"] = 389577,
		["SoothingMist"] = 115175,
		["EyeoftheTiger"] = 196607,
		["HastyProvocation"] = 328670,
		["TigerTailSweep"] = 264348,
		["CraneVortex"] = 388848,
		["ElusiveMists"] = 388681,
		["CracklingJadeLightning"] = 117952,
		["Windwalking"] = 157411,
		["Thunderfist"] = 392985,
		["SummonBlackOxStatue"] = 115315,
		["ImprovedRoll"] = 328669,
		["Attenuation"] = 386941,
		["HardenedSoles"] = 391383,
		["WideningWhirl"] = 388846,
		["VigorousExpulsion"] = 392900,
		["Roll"] = 109132,
		["Afterlife"] = 116092,
		["LegSweep"] = 119381,
		["StormEarthandFire"] = 137639,
		["RushingJadeWind"] = 116847,
		["ExpeditiousFortification"] = 388813,
		["JadeIgnition"] = 392979,
		["ExpelHarm"] = 322101,
		["ImprovedVivify"] = 231602,
		["SaveThemAll"] = 389579,
		["SummonJadeSerpentStatue"] = 115313,
		["TouchofDeath"] = 322109,
		["ChiTorpedo"] = 115008,
		["Resuscitate"] = 115178,
		["WhirlingDragonPunch"] = 152175,
		["ChiBurst"] = 123986,
		["Vivify"] = 116670,
		["StrikeoftheWindlord"] = 392983,
		["Detox"] = 218164,
		["DanceofChiJi"] = 325201,
		["OpenPalmStrikes"] = 392970,
		["BlackoutKick"] = 100784,
		["YulonsGrace"] = 414131,
		["MasteryComboStrikes"] = 115636,
		["SpiritualFocus"] = 280197,
		["EmpoweredTigerLightning"] = 323999,
		["FlashingFists"] = 388854,
		["SummonWhiteTigerStatue"] = 388686,
		["EscapefromReality"] = 394110,
		["IronshellBrew"] = 388814,
		["Provoke"] = 115546,
		["GloryoftheDawn"] = 392958,
		["TranscendenceTransfer"] = 119996,
		["DustintheWind"] = 394093,
		["Celerity"] = 115173,
		["Skytouch"] = 405044,
		["FlyingSerpentKick"] = 101545,
		["TigerPalm"] = 100780,
		["FerocityofXuen"] = 388674,
		["BonedustBrew"] = 386276,
		["ClosetoHeart"] = 389574,
		["DampenHarm"] = 122278,
		["Disable"] = 116095,
		["SongofChiJi"] = 198898,
		["MeridianStrikes"] = 391330,
		["RisingStar"] = 388849,
		["VivaciousVivification"] = 388812,
		["PathofJade"] = 392994,
		["ImprovedTouchofDeath"] = 322113,
		["ImprovedParalysis"] = 344359,
		["RingofPeace"] = 116844,
		["FatalFlyingGuillotine"] = 394923,
		["SpearHandStrike"] = 116705,
		["DiffuseMagic"] = 122783,
		["ForbiddenTechnique"] = 393098,
		["FistsofFury"] = 113656,
		["FuryofXuen"] = 396166,
		["ShadowboxingTreads"] = 392982,
		["InvokeXuentheWhiteTiger"] = 123904,
		["TigersLust"] = 116841,
		["RisingSunKick"] = 107428,
		["XuensBattlegear"] = 392993,
		["ChiWave"] = 115098,
		["TeachingsoftheMonastery"] = 116645,
		["FastFeet"] = 388809,
		["HitCombo"] = 196740,
		["InnerPeace"] = 397768,
		["MysticTouch"] = 8647,
		["PowerStrikes"] = 121817,
		["FatalTouch"] = 394123,
		["TransferthePower"] = 195300,
		["Paralysis"] = 115078,
		["ResonantFists"] = 389578,
		["SpinningCraneKick"] = 101546,
		["TouchofKarma"] = 122470,
		},
		["Brewmaster"] = {
		["CalmingPresence"] = 388664,
		["GracefulExit"] = 387256,
		["GenerousPour"] = 389575,
		["CelestialBrew"] = 322507,
		["MysticTouch"] = 8647,
		["FortifyingBrew"] = 115203,
		["ImprovedParalysis"] = 344359,
		["ElusiveMists"] = 388681,
		["HighTolerance"] = 196737,
		["HealingElixir"] = 122281,
		["KegSmash"] = 121253,
		["ZenFlight"] = 125883,
		["ZenPilgrimage"] = 126892,
		["CelestialFlames"] = 325177,
		["VivaciousVivification"] = 388812,
		["BounceBack"] = 389577,
		["HastyProvocation"] = 328670,
		["QuickSip"] = 388505,
		["CharredPassions"] = 386965,
		["StrengthofSpirit"] = 387276,
		["ProfoundRebuttal"] = 392910,
		["Transcendence"] = 101643,
		["Detox"] = 218164,
		["WalkwiththeOx"] = 387219,
		["DanceoftheWind"] = 414132,
		["ImprovedInvokeNiuzaotheBlackOx"] = 322740,
		["PresstheAdvantage"] = 418359,
		["GraceoftheCrane"] = 388811,
		["BobandWeave"] = 280515,
		["SoothingMist"] = 115175,
		["EyeoftheTiger"] = 196607,
		["BountifulBrew"] = 386949,
		["ImprovedCelestialBrew"] = 322510,
		["Roll"] = 109132,
		["RushingJadeWind"] = 116847,
		["LegSweep"] = 119381,
		["Windwalking"] = 157411,
		["ScaldingBrew"] = 383698,
		["SpiritoftheOx"] = 400629,
		["ImprovedRoll"] = 328669,
		["Attenuation"] = 386941,
		["FortifyingBrewDetermination"] = 322960,
		["ChiWave"] = 115098,
		["BlackOxBrew"] = 115399,
		["ChiTorpedo"] = 115008,
		["Vivify"] = 116670,
		["GaiPlinsImperialBrew"] = 383700,
		["Stagger"] = 115069,
		["ZenMeditation"] = 115176,
		["ExpeditiousFortification"] = 388813,
		["DiffuseMagic"] = 122783,
		["ExpelHarm"] = 322101,
		["ImprovedVivify"] = 231602,
		["DragonfireBrew"] = 383994,
		["BrewmastersBalance"] = 245013,
		["TouchofDeath"] = 322109,
		["BonedustBrew"] = 386276,
		["Resuscitate"] = 115178,
		["PretenseofInstability"] = 393516,
		["Clash"] = 324312,
		["MasteryElusiveBrawler"] = 117906,
		["SummonWhiteTigerStatue"] = 388686,
		["HitScheme"] = 383695,
		["RingofPeace"] = 116844,
		["BlackoutKick"] = 100784,
		["StormstoutsLastKeg"] = 383707,
		["ShadowboxingTreads"] = 387638,
		["TrainingofNiuzao"] = 383714,
		["ExplodingKeg"] = 325153,
		["TigerTailSweep"] = 264348,
		["SummonBlackOxStatue"] = 115315,
		["CelestialFortune"] = 216519,
		["TranquilSpirit"] = 393357,
		["TranscendenceTransfer"] = 119996,
		["GiftoftheOx"] = 124502,
		["SpecialDelivery"] = 196730,
		["Counterstrike"] = 383785,
		["ChiSurge"] = 393400,
		["Celerity"] = 115173,
		["FastFeet"] = 388809,
		["SaveThemAll"] = 389579,
		["TigerPalm"] = 100780,
		["Provoke"] = 115546,
		["SummonJadeSerpentStatue"] = 115313,
		["ClosetoHeart"] = 389574,
		["Shuffle"] = 322120,
		["Disable"] = 116095,
		["SongofChiJi"] = 198898,
		["DampenHarm"] = 122278,
		["Anvil&Stave"] = 386937,
		["VigorousExpulsion"] = 392900,
		["ImprovedTouchofDeath"] = 322113,
		["ChiBurst"] = 123986,
		["FerocityofXuen"] = 388674,
		["PurifyingBrew"] = 119582,
		["StaggeringStrikes"] = 387625,
		["SpearHandStrike"] = 116705,
		["FacePalm"] = 389942,
		["ElusiveFootwork"] = 387046,
		["EscapefromReality"] = 394110,
		["WeaponsofOrder"] = 387184,
		["FluidityofMotion"] = 387230,
		["YulonsGrace"] = 414131,
		["TigersLust"] = 116841,
		["RisingSunKick"] = 107428,
		["BlackoutCombo"] = 196736,
		["FundamentalObservation"] = 387035,
		["SalsalabimsStrength"] = 383697,
		["BreathofFire"] = 115181,
		["ImprovedPurifyingBrew"] = 343743,
		["InvokeNiuzaotheBlackOx"] = 132578,
		["CalltoArms"] = 397251,
		["LightBrewing"] = 325093,
		["FatalTouch"] = 394123,
		["CracklingJadeLightning"] = 117952,
		["Paralysis"] = 115078,
		["ResonantFists"] = 389578,
		["SpinningCraneKick"] = 101546,
		["IronshellBrew"] = 388814,
		},
		["Mistweaver"] = {
		["CalmingPresence"] = 388664,
		["Revival"] = 115310,
		["GenerousPour"] = 389575,
		["ZenPulse"] = 124081,
		["Upwelling"] = 274963,
		["AwakenedJadefire"] = 388779,
		["FortifyingBrew"] = 115203,
		["ImprovedParalysis"] = 344359,
		["RefreshingJadeWind"] = 196725,
		["ElusiveMists"] = 388681,
		["LegacyofWisdom"] = 404408,
		["ZenFlight"] = 125883,
		["Transcendence"] = 101643,
		["FontofLife"] = 337209,
		["JadefireStomp"] = 388193,
		["SpinningCraneKick"] = 101546,
		["BounceBack"] = 389577,
		["MistyPeaks"] = 388682,
		["ResplendentMist"] = 388020,
		["InvokersDelight"] = 388661,
		["NourishingChi"] = 387765,
		["HastyProvocation"] = 328670,
		["MysticTouch"] = 8647,
		["StrengthofSpirit"] = 387276,
		["GiftoftheCelestials"] = 388212,
		["ProfoundRebuttal"] = 392910,
		["RapidDiffusion"] = 388847,
		["CalmingCoalescence"] = 388218,
		["CracklingJadeLightning"] = 117952,
		["RenewingMist"] = 115151,
		["DanceoftheWind"] = 432181,
		["CelestialHarmony"] = 343655,
		["GraceoftheCrane"] = 388811,
		["SummonJadeSerpentStatue"] = 115313,
		["SummonBlackOxStatue"] = 115315,
		["Restoral"] = 388615,
		["Roll"] = 109132,
		["BurstofLife"] = 399226,
		["SaveThemAll"] = 389579,
		["VeilofPride"] = 400053,
		["TearofMorning"] = 387991,
		["UpliftedSpirits"] = 388551,
		["CloudedFocus"] = 388047,
		["YulonsWhisper"] = 388038,
		["ImprovedRoll"] = 328669,
		["InvigoratingMists"] = 274586,
		["DancingMists"] = 388701,
		["ChiWave"] = 115098,
		["VigorousExpulsion"] = 392900,
		["LegSweep"] = 119381,
		["Reawaken"] = 212051,
		["BlackoutKick"] = 100784,
		["MendingProliferation"] = 388509,
		["InvokeChiJitheRedCrane"] = 325197,
		["ExpeditiousFortification"] = 388813,
		["ShaohaosLessons"] = 400089,
		["ExpelHarm"] = 322101,
		["ThunderFocusTea"] = 116680,
		["SheilunsGift"] = 399491,
		["PeacefulMending"] = 388593,
		["TeaofPlenty"] = 388517,
		["ChiTorpedo"] = 115008,
		["Resuscitate"] = 115178,
		["Lifecycles"] = 197915,
		["EscapefromReality"] = 394110,
		["Vivify"] = 116670,
		["SummonWhiteTigerStatue"] = 388686,
		["EnergizingBrew"] = 422031,
		["DanceofChiJi"] = 438439,
		["ChiBurst"] = 123986,
		["EchoingReverberation"] = 388604,
		["MistWrap"] = 197900,
		["RingofPeace"] = 116844,
		["HealingElixir"] = 122280,
		["ImprovedVivify"] = 231602,
		["PeerIntoPeace"] = 440008,
		["FocusedThunder"] = 197895,
		["EyeoftheTiger"] = 196607,
		["Windwalking"] = 157411,
		["Detox"] = 115450,
		["DampenHarm"] = 122278,
		["TranscendenceTransfer"] = 119996,
		["Celerity"] = 115173,
		["MasteryGustofMists"] = 117907,
		["RisingMist"] = 274909,
		["Provoke"] = 115546,
		["TigerPalm"] = 100780,
		["FerocityofXuen"] = 388674,
		["AccumulatingMist"] = 388564,
		["ClosetoHeart"] = 389574,
		["MistsofLife"] = 388548,
		["Disable"] = 116095,
		["SongofChiJi"] = 198898,
		["TeaofSerenity"] = 393460,
		["JadeBond"] = 388031,
		["VivaciousVivification"] = 388812,
		["ImprovedTouchofDeath"] = 322113,
		["Chrysalis"] = 202424,
		["ManaTea"] = 115294,
		["SecretInfusion"] = 388491,
		["FastFeet"] = 388809,
		["SpearHandStrike"] = 116705,
		["AncientConcordance"] = 388740,
		["Unison"] = 388477,
		["OverflowingMists"] = 388511,
		["DiffuseMagic"] = 122783,
		["TouchofDeath"] = 322109,
		["YulonsGrace"] = 414131,
		["TigersLust"] = 116841,
		["RisingSunKick"] = 107428,
		["LifeCocoon"] = 116849,
		["AncientTeachings"] = 388023,
		["ImprovedDetox"] = 388874,
		["TigerTailSweep"] = 264348,
		["SoothingMist"] = 115175,
		["ZenPilgrimage"] = 126892,
		["TeachingsoftheMonastery"] = 116645,
		["EnvelopingMist"] = 124682,
		["FatalTouch"] = 394123,
		["EssenceFont"] = 191837,
		["Paralysis"] = 115078,
		["ResonantFists"] = 389578,
		["InvokeYulontheJadeSerpent"] = 322118,
		["IronshellBrew"] = 388814,
		},
		},
		["Rogue"] = {
		["Outlaw"] = {
		["Ruthlessness"] = 14161,
		["Evasion"] = 5277,
		["KeepItRolling"] = 381989,
		["SuperiorMixture"] = 423701,
		["PickPocket"] = 921,
		["ImprovedMainGauche"] = 382746,
		["Featherfoot"] = 423683,
		["ShadowDance"] = 185313,
		["HeavyHitter"] = 381885,
		["FleetFooted"] = 378813,
		["BlindingPowder"] = 256165,
		["GracefulGuile"] = 423647,
		["EchoingReprimand"] = 385616,
		["Blind"] = 2094,
		["PistolShot"] = 185763,
		["DirtyTricks"] = 108216,
		["VirulentPoisons"] = 381543,
		["Audacity"] = 381845,
		["TakeembySurprise"] = 382742,
		["TripleThreat"] = 381894,
		["ImprovedAdrenalineRush"] = 395422,
		["TightSpender"] = 381621,
		["Riposte"] = 344363,
		["FloatLikeaButterfly"] = 354897,
		["SwiftSlasher"] = 381988,
		["CheapShot"] = 1833,
		["Kick"] = 1766,
		["SafeFall"] = 1860,
		["BladeRush"] = 271877,
		["BetweentheEyes"] = 315341,
		["ResoundingClarity"] = 381622,
		["UnderhandedUpperHand"] = 424044,
		["HitandRun"] = 196922,
		["Blackjack"] = 379005,
		["Ambidexterity"] = 381822,
		["RetractableHook"] = 256188,
		["DeadlyPrecision"] = 381542,
		["SummarilyDispatched"] = 381990,
		["NimbleFingers"] = 378427,
		["Crackshot"] = 423703,
		["TricksoftheTrade"] = 57934,
		["ImprovedAmbush"] = 381620,
		["Shiv"] = 5938,
		["Sepsis"] = 385408,
		["Detection"] = 56814,
		["AcrobaticStrikes"] = 196924,
		["LeechingPoison"] = 280716,
		["SliceandDice"] = 315496,
		["Sap"] = 6770,
		["SinisterStrike"] = 1752,
		["Alacrity"] = 193539,
		["DeftManeuvers"] = 381878,
		["KillingSpree"] = 51690,
		["ThistleTea"] = 381623,
		["SleightofHand"] = 381839,
		["Vanish"] = 1856,
		["StingLikeaBee"] = 131511,
		["AtrophicPoison"] = 381637,
		["AceUpYourSleeve"] = 381828,
		["PrecisionShot"] = 428377,
		["ImprovedSprint"] = 231691,
		["Vigor"] = 14983,
		["CheatDeath"] = 31230,
		["GrapplingHook"] = 195457,
		["GhostlyStrike"] = 196937,
		["Reverberation"] = 394332,
		["PickLock"] = 1804,
		["HiddenOpportunity"] = 383281,
		["NumbingPoison"] = 5761,
		["BladeFlurry"] = 13877,
		["ShroudofConcealment"] = 114018,
		["Ambush"] = 8676,
		["GreenskinsWickers"] = 386823,
		["LoadedDice"] = 256170,
		["CloakofShadows"] = 31224,
		["Lethality"] = 382238,
		["RestlessBlades"] = 79096,
		["SoothingDarkness"] = 393970,
		["CrimsonVial"] = 185311,
		["ColdBlood"] = 382245,
		["ImprovedBetweentheEyes"] = 235484,
		["MasterPoisoner"] = 378436,
		["Nightstalker"] = 14062,
		["AirborneIrritant"] = 200733,
		["ImprovedWoundPoison"] = 319066,
		["CombatStamina"] = 381877,
		["Sprint"] = 2983,
		["RolltheBones"] = 315508,
		["UnbreakableStride"] = 400804,
		["Recuperator"] = 378996,
		["DeeperStratagem"] = 193531,
		["DancingSteel"] = 272026,
		["RushedSetup"] = 378803,
		["MasteryMainGauche"] = 76806,
		["Eviscerate"] = 196819,
		["Shadowstep"] = 36554,
		["Stillshroud"] = 423662,
		["IronStomach"] = 193546,
		["Dispatch"] = 2098,
		["Elusiveness"] = 79008,
		["DeadenedNerves"] = 231719,
		["AdrenalineRush"] = 13750,
		["PreciseCuts"] = 381985,
		["DeviousStratagem"] = 394321,
		["QuickDraw"] = 196938,
		["CombatPotency"] = 61329,
		["Gouge"] = 1776,
		["FatalFlourish"] = 35551,
		["Shadowrunner"] = 378807,
		["Distract"] = 1725,
		["CounttheOdds"] = 381982,
		["Feint"] = 1966,
		["Opportunity"] = 279876,
		["KidneyShot"] = 408,
		["ThiefsVersatility"] = 381619,
		["Subterfuge"] = 108208,
		["FantheHammer"] = 381846,
		["Stealth"] = 1784,
		},
		["Subtlety"] = {
		["Inevitability"] = 382512,
		["Evasion"] = 5277,
		["SuperiorMixture"] = 423701,
		["ImprovedShurikenStorm"] = 319951,
		["Featherfoot"] = 423683,
		["DarkShadow"] = 245687,
		["FleetFooted"] = 378813,
		["PickPocket"] = 921,
		["Shadowstrike"] = 185438,
		["DarkBrew"] = 382504,
		["SafeFall"] = 1860,
		["Finality"] = 382525,
		["Subterfuge"] = 108208,
		["Blind"] = 2094,
		["Feint"] = 1966,
		["Kick"] = 1766,
		["TerrifyingPace"] = 428387,
		["Weaponmaster"] = 193537,
		["VirulentPoisons"] = 381543,
		["Vigor"] = 14983,
		["ShadowDance"] = 185313,
		["DeadlyPrecision"] = 381542,
		["Veiltouched"] = 382017,
		["PlannedExecution"] = 382508,
		["Stealth"] = 1784,
		["TightSpender"] = 381621,
		["GoremawsBite"] = 426591,
		["MasteryExecutioner"] = 76808,
		["Distract"] = 1725,
		["KidneyShot"] = 408,
		["Backstab"] = 53,
		["Detection"] = 56814,
		["LeechingPoison"] = 280716,
		["ShurikenToss"] = 114014,
		["Shadowrunner"] = 378807,
		["ResoundingClarity"] = 381622,
		["ImprovedAmbush"] = 381620,
		["EphemeralBond"] = 426563,
		["WithoutaTrace"] = 382513,
		["ShadowTechniques"] = 196912,
		["CheatDeath"] = 31230,
		["ShadowBlades"] = 121471,
		["ShotintheDark"] = 257505,
		["CloakedinShadows"] = 382515,
		["TheRotten"] = 382015,
		["TricksoftheTrade"] = 57934,
		["ExhilaratingExecution"] = 428486,
		["Shiv"] = 5938,
		["Sepsis"] = 385408,
		["DanseMacabre"] = 382528,
		["ReplicatingShadows"] = 382506,
		["Flagellation"] = 384631,
		["GracefulGuile"] = 423647,
		["Sap"] = 6770,
		["Elusiveness"] = 79008,
		["Alacrity"] = 193539,
		["PerforatedVeins"] = 382518,
		["DeepeningShadows"] = 185314,
		["AcrobaticStrikes"] = 196924,
		["ThistleTea"] = 381623,
		["Vanish"] = 1856,
		["Stillshroud"] = 423662,
		["SymbolsofDeath"] = 212283,
		["ImprovedShadowTechniques"] = 394023,
		["SwiftDeath"] = 394309,
		["ImprovedSprint"] = 231691,
		["ShurikenTornado"] = 277925,
		["ImprovedShadowDance"] = 393972,
		["Reverberation"] = 394332,
		["Rupture"] = 1943,
		["UnbreakableStride"] = 400804,
		["CloakofShadows"] = 31224,
		["FadetoNothing"] = 382514,
		["Gloomblade"] = 200758,
		["ImprovedBackstab"] = 319949,
		["NumbingPoison"] = 5761,
		["Ambush"] = 8676,
		["Lethality"] = 382238,
		["ShadowFocus"] = 108209,
		["SecretTechnique"] = 280719,
		["WarningSigns"] = 426555,
		["ColdBlood"] = 382245,
		["SoothingDarkness"] = 393970,
		["RushedSetup"] = 378803,
		["Shadowcraft"] = 426594,
		["Sprint"] = 2983,
		["MasterPoisoner"] = 378436,
		["Nightstalker"] = 14062,
		["AirborneIrritant"] = 200733,
		["ImprovedWoundPoison"] = 319066,
		["AtrophicPoison"] = 381637,
		["NightTerrors"] = 277953,
		["ShurikenStorm"] = 197835,
		["CrimsonVial"] = 185311,
		["Recuperator"] = 378996,
		["DeeperStratagem"] = 193531,
		["ShroudofConcealment"] = 114018,
		["TheFirstDance"] = 382505,
		["NimbleFingers"] = 378427,
		["Eviscerate"] = 196819,
		["Shadowstep"] = 36554,
		["InvigoratingShadowdust"] = 382523,
		["QuickDecisions"] = 382503,
		["MasterofShadows"] = 196976,
		["RelentlessStrikes"] = 58423,
		["DeadenedNerves"] = 231719,
		["ShadowedFinishers"] = 382511,
		["IronStomach"] = 193546,
		["DeeperDaggers"] = 382517,
		["SinisterStrike"] = 1752,
		["SliceandDice"] = 315496,
		["Gouge"] = 1776,
		["FindWeakness"] = 91023,
		["CheapShot"] = 1833,
		["LingeringShadow"] = 382524,
		["Blackjack"] = 379005,
		["Premeditation"] = 343160,
		["EchoingReprimand"] = 385616,
		["SilentStorm"] = 385722,
		["ShroudedinDarkness"] = 382507,
		["SecretStratagem"] = 394320,
		["PickLock"] = 1804,
		["BlackPowder"] = 319175,
		},
		["Assassination"] = {
		["AtrophicPoison"] = 381637,
		["Evasion"] = 5277,
		["SystemicFailure"] = 381652,
		["SuperiorMixture"] = 423701,
		["PickPocket"] = 921,
		["VenomRush"] = 152152,
		["Featherfoot"] = 423683,
		["ShadowDance"] = 185313,
		["CuttotheChase"] = 51667,
		["FleetFooted"] = 378813,
		["BloodyMess"] = 381626,
		["GracefulGuile"] = 423647,
		["Subterfuge"] = 108208,
		["Blind"] = 2094,
		["LethalDose"] = 381640,
		["VirulentPoisons"] = 381543,
		["Vigor"] = 14983,
		["DragonTemperedBlades"] = 381801,
		["SealFate"] = 14190,
		["FlyingDaggers"] = 381631,
		["Stealth"] = 1784,
		["TightSpender"] = 381621,
		["IronWire"] = 196861,
		["PoisonedKnife"] = 185565,
		["CheapShot"] = 1833,
		["ThrownPrecision"] = 381629,
		["ScentofBlood"] = 381799,
		["ResoundingClarity"] = 381622,
		["TwisttheKnife"] = 381669,
		["Shadowrunner"] = 378807,
		["CausticSpatter"] = 421975,
		["SerratedBoneSpike"] = 385424,
		["CheatDeath"] = 31230,
		["ImprovedGarrote"] = 381632,
		["Sepsis"] = 385408,
		["NimbleFingers"] = 378427,
		["Kick"] = 1766,
		["TricksoftheTrade"] = 57934,
		["ImprovedAmbush"] = 381620,
		["AmplifyingPoison"] = 381664,
		["IronStomach"] = 193546,
		["DashingScoundrel"] = 381797,
		["AcrobaticStrikes"] = 196924,
		["Kingsbane"] = 385627,
		["PickLock"] = 1804,
		["Sap"] = 6770,
		["ShroudedSuffocation"] = 385478,
		["Alacrity"] = 193539,
		["ZoldyckRecipe"] = 381798,
		["MasteryPotentAssassin"] = 76803,
		["ImprovedShiv"] = 319032,
		["ThistleTea"] = 381623,
		["Vanish"] = 1856,
		["KidneyShot"] = 408,
		["Garrote"] = 703,
		["Feint"] = 1966,
		["Envenom"] = 32645,
		["ImprovedSprint"] = 231691,
		["LightweightShiv"] = 394983,
		["Gouge"] = 1776,
		["UnbreakableStride"] = 400804,
		["Rupture"] = 1943,
		["Reverberation"] = 394332,
		["FatalConcoction"] = 392384,
		["SinisterStrike"] = 1752,
		["IntenttoKill"] = 381630,
		["ImprovedPoisons"] = 381624,
		["ShroudofConcealment"] = 114018,
		["Ambush"] = 8676,
		["EchoingReprimand"] = 385616,
		["Mutilate"] = 1329,
		["CloakofShadows"] = 31224,
		["Lethality"] = 382238,
		["CrimsonVial"] = 185311,
		["SoothingDarkness"] = 393970,
		["RushedSetup"] = 378803,
		["AirborneIrritant"] = 200733,
		["ColdBlood"] = 382245,
		["MasterPoisoner"] = 378436,
		["Nightstalker"] = 14062,
		["Deathmark"] = 360194,
		["ImprovedWoundPoison"] = 319066,
		["PathofBlood"] = 423054,
		["Sprint"] = 2983,
		["Blindside"] = 328085,
		["Stillshroud"] = 423662,
		["Recuperator"] = 378996,
		["DeeperStratagem"] = 193531,
		["ArterialPrecision"] = 400783,
		["Eviscerate"] = 196819,
		["DeadlyPoison"] = 2823,
		["PoisonBomb"] = 255544,
		["Shadowstep"] = 36554,
		["MasterAssassin"] = 255989,
		["SanguineBlades"] = 200806,
		["Doomblade"] = 381673,
		["Elusiveness"] = 79008,
		["DeadenedNerves"] = 231719,
		["IndiscriminateCarnage"] = 381802,
		["ViciousVenoms"] = 381634,
		["NumbingPoison"] = 5761,
		["CrimsonTempest"] = 121411,
		["SliceandDice"] = 315496,
		["LeechingPoison"] = 280716,
		["Detection"] = 56814,
		["TinyToxicBlade"] = 381800,
		["Distract"] = 1725,
		["Blackjack"] = 379005,
		["SuddenDemise"] = 423136,
		["DeadlyPrecision"] = 381542,
		["InternalBleeding"] = 381627,
		["VenomousWounds"] = 79134,
		["Shiv"] = 5938,
		["FanofKnives"] = 51723,
		["SafeFall"] = 1860,
		},
		},
		["Mage"] = {
		["Frost"] = {
		["FracturedFrost"] = 378448,
		["RemoveCurse"] = 475,
		["TomeofAntonidas"] = 382490,
		["TomeofRhonin"] = 382493,
		["Polymorph"] = 118,
		["DivertedEnergy"] = 382270,
		["Flurry"] = 44614,
		["IncantationofSwiftness"] = 382293,
		["BrainFreeze"] = 190447,
		["PiercingCold"] = 378919,
		["MassPolymorph"] = 383121,
		["SplinteringRay"] = 418733,
		["ArcaneWarding"] = 383092,
		["Blink"] = 1953,
		["RayofFrost"] = 205021,
		["TimeManipulation"] = 387807,
		["DragonsBreath"] = 31661,
		["IceCold"] = 414658,
		["Spellsteal"] = 30449,
		["GlacialAssault"] = 378947,
		["IceBarrier"] = 11426,
		["Shatter"] = 12982,
		["IceNova"] = 157997,
		["PerpetualWinter"] = 378198,
		["Slow"] = 31589,
		["OverflowingEnergy"] = 390218,
		["SlickIce"] = 382144,
		["MasterofTime"] = 342249,
		["Blizzard"] = 190356,
		["SplinteringCold"] = 379049,
		["WintersProtection"] = 382424,
		["FireBlast"] = 319836,
		["CryoFreeze"] = 382292,
		["FrigidWinds"] = 235224,
		["FrostNova"] = 122,
		["SlowFall"] = 130,
		["IceFloes"] = 108839,
		["FlowofTime"] = 382268,
		["EnergizedBarriers"] = 386828,
		["ConjureRefreshment"] = 190336,
		["LonelyWinter"] = 205024,
		["VolatileDetonation"] = 389627,
		["ArcaneIntellect"] = 1459,
		["Reabsorption"] = 382820,
		["TimeAnomaly"] = 383243,
		["ArcaneExplosion"] = 1449,
		["DeepShatter"] = 378749,
		["IceCaller"] = 236662,
		["ColdSnap"] = 235219,
		["QuickWitted"] = 382297,
		["FrozenOrb"] = 84714,
		["FrozenTouch"] = 205030,
		["IceWard"] = 205036,
		["IncantersFlow"] = 1463,
		["Subzero"] = 380154,
		["MassBarrier"] = 414660,
		["TempestBarrier"] = 382289,
		["ShiftingPower"] = 382440,
		["WintersBlessing"] = 417489,
		["TemporalWarp"] = 386539,
		["IceBlock"] = 45438,
		["EverlastingFrost"] = 385167,
		["FreezingRain"] = 270233,
		["Cryopathy"] = 417491,
		["AccumulativeShielding"] = 382800,
		["TemporalVelocity"] = 382826,
		["CometStorm"] = 153595,
		["Hailstones"] = 381244,
		["MasteryIcicles"] = 76613,
		["SummonWaterElemental"] = 31687,
		["Reduplication"] = 382569,
		["ImprovedFrostNova"] = 343183,
		["RigidIce"] = 382481,
		["ConeofCold"] = 120,
		["FingersofFrost"] = 112965,
		["MirrorImage"] = 55342,
		["FreezingWinds"] = 382103,
		["MassInvisibility"] = 414664,
		["GlacialSpike"] = 199786,
		["FlashFreeze"] = 379993,
		["TimeWarp"] = 80353,
		["Snowstorm"] = 381706,
		["Frostbolt"] = 116,
		["IcyVeins"] = 12472,
		["Wintertide"] = 378406,
		["AlterTime"] = 342245,
		["IceLance"] = 30455,
		["SplittingIce"] = 56377,
		["ThermalVoid"] = 155149,
		["Frostbite"] = 378756,
		["ChainReaction"] = 278309,
		["GreaterInvisibility"] = 110959,
		["Invisibility"] = 66,
		["ColdFront"] = 382110,
		["RingofFrost"] = 113724,
		["BoneChilling"] = 205027,
		["BlastWave"] = 157981,
		["FreezingCold"] = 386763,
		["Shimmer"] = 212653,
		["MassSlow"] = 391102,
		["Counterspell"] = 2139,
		["ColdestSnap"] = 417493,
		["Displacement"] = 389713,
		},
		["Fire"] = {
		["RemoveCurse"] = 475,
		["TomeofAntonidas"] = 382490,
		["TomeofRhonin"] = 382493,
		["CriticalMass"] = 117216,
		["Polymorph"] = 118,
		["DivertedEnergy"] = 382270,
		["IncantationofSwiftness"] = 382293,
		["DeepImpact"] = 416719,
		["FeeltheBurn"] = 383391,
		["RigidIce"] = 382481,
		["TempestBarrier"] = 382289,
		["ArcaneWarding"] = 383092,
		["Blink"] = 1953,
		["TimeManipulation"] = 387807,
		["DragonsBreath"] = 31661,
		["AlexstraszasFury"] = 235870,
		["IceCold"] = 414659,
		["Fireball"] = 133,
		["Spellsteal"] = 30449,
		["ImprovedScorch"] = 383604,
		["Conflagration"] = 205023,
		["Pyromaniac"] = 205020,
		["FueltheFire"] = 416094,
		["MirrorImage"] = 55342,
		["Meteor"] = 153561,
		["SunKingsBlessing"] = 383886,
		["ConjureRefreshment"] = 190336,
		["Slow"] = 31589,
		["OverflowingEnergy"] = 390218,
		["HotStreak"] = 195283,
		["WintersProtection"] = 382424,
		["MasterofTime"] = 342249,
		["Reabsorption"] = 382820,
		["Pyrotechnics"] = 157642,
		["SearingTouch"] = 269644,
		["EnergizedBarriers"] = 386828,
		["FireBlast"] = 319836,
		["CryoFreeze"] = 382292,
		["FlamePatch"] = 205037,
		["IceBlock"] = 45438,
		["SlowFall"] = 130,
		["IceFloes"] = 108839,
		["TemperedFlames"] = 383659,
		["Invisibility"] = 66,
		["FrostNova"] = 122,
		["FreezingCold"] = 386763,
		["VolatileDetonation"] = 389627,
		["Kindling"] = 155148,
		["Counterspell"] = 2139,
		["TimeAnomaly"] = 383243,
		["IceNova"] = 157997,
		["BlastWave"] = 157981,
		["IncendiaryEruptions"] = 383665,
		["Pyroblast"] = 11366,
		["QuickWitted"] = 382297,
		["BlazingBarrier"] = 235313,
		["MasterofFlame"] = 384174,
		["IceWard"] = 205036,
		["IncantersFlow"] = 1463,
		["MassPolymorph"] = 383121,
		["MassBarrier"] = 414660,
		["LivingBomb"] = 44457,
		["ShiftingPower"] = 382440,
		["Convection"] = 416715,
		["TemporalWarp"] = 386539,
		["Combustion"] = 190319,
		["FerventFlickering"] = 387044,
		["RingofFrost"] = 113724,
		["TemporalVelocity"] = 382826,
		["FrigidWinds"] = 235224,
		["AccumulativeShielding"] = 382800,
		["AlterTime"] = 342245,
		["FeveredIncantation"] = 383810,
		["FlameAccelerant"] = 203275,
		["UnleashedInferno"] = 416506,
		["Firefall"] = 384033,
		["ImprovedFrostNova"] = 343183,
		["PhoenixReborn"] = 383476,
		["ConeofCold"] = 120,
		["MassInvisibility"] = 414664,
		["FieryRush"] = 383634,
		["Frostbolt"] = 116,
		["Firestarter"] = 205026,
		["Wildfire"] = 383489,
		["MasteryIgnite"] = 12846,
		["TimeWarp"] = 80353,
		["IntensifyingFlame"] = 416714,
		["Reduplication"] = 382569,
		["Scorch"] = 2948,
		["Flamestrike"] = 2120,
		["PhoenixFlames"] = 257541,
		["CalloftheSunKing"] = 343222,
		["SurgingBlaze"] = 343230,
		["GreaterInvisibility"] = 110959,
		["FromtheAshes"] = 342344,
		["Inflame"] = 417467,
		["Firemind"] = 383499,
		["FlameOn"] = 205029,
		["Hyperthermia"] = 383860,
		["ArcaneExplosion"] = 1449,
		["ArcaneIntellect"] = 1459,
		["ControlledDestruction"] = 383669,
		["Cauterize"] = 86949,
		["Shimmer"] = 212653,
		["MassSlow"] = 391102,
		["FlowofTime"] = 382268,
		["ImprovedCombustion"] = 383967,
		["Displacement"] = 389713,
		},
		["Arcane"] = {
		["ConjureManaGem"] = 759,
		["RemoveCurse"] = 475,
		["TomeofAntonidas"] = 382490,
		["TomeofRhonin"] = 382493,
		["ArcaneFamiliar"] = 205022,
		["Polymorph"] = 118,
		["DivertedEnergy"] = 382270,
		["OrbBarrage"] = 384858,
		["RuleofThrees"] = 264354,
		["IncantationofSwiftness"] = 382293,
		["ArcaneBlast"] = 30451,
		["ProdigiousSavant"] = 384612,
		["RigidIce"] = 382481,
		["TempestBarrier"] = 382289,
		["ArcaneWarding"] = 383092,
		["NetherTempest"] = 114923,
		["TimeManipulation"] = 387807,
		["DragonsBreath"] = 31661,
		["IceCold"] = 414659,
		["Spellsteal"] = 30449,
		["ChronoShift"] = 235711,
		["ChargedOrb"] = 384651,
		["IceNova"] = 157997,
		["Concentration"] = 384374,
		["ArcaneHarmony"] = 384452,
		["Slow"] = 31589,
		["OverflowingEnergy"] = 390218,
		["ArcaneMissiles"] = 5143,
		["IlluminatedThoughts"] = 384060,
		["MasterofTime"] = 342249,
		["Enlightened"] = 321387,
		["Displacement"] = 389713,
		["ArcaneSurge"] = 365350,
		["FireBlast"] = 319836,
		["CryoFreeze"] = 382292,
		["FrigidWinds"] = 235224,
		["IceBlock"] = 45438,
		["NetherPrecision"] = 383782,
		["IceFloes"] = 108839,
		["FlowofTime"] = 382268,
		["EnergizedBarriers"] = 386828,
		["ArcaneEcho"] = 342231,
		["Amplification"] = 236628,
		["VolatileDetonation"] = 389627,
		["SlowFall"] = 130,
		["Invisibility"] = 66,
		["TimeAnomaly"] = 383243,
		["ArcingCleave"] = 231564,
		["Reverberate"] = 281482,
		["FrostNova"] = 122,
		["ArcaneBarrage"] = 44425,
		["QuickWitted"] = 382297,
		["RingofFrost"] = 113724,
		["CascadingPower"] = 384276,
		["IceWard"] = 205036,
		["IncantersFlow"] = 1463,
		["MirrorImage"] = 55342,
		["MassBarrier"] = 414660,
		["Reabsorption"] = 382820,
		["ShiftingPower"] = 382440,
		["Clearcasting"] = 79684,
		["TemporalWarp"] = 386539,
		["BlastWave"] = 157981,
		["WintersProtection"] = 382424,
		["Blink"] = 1953,
		["TemporalVelocity"] = 382826,
		["PrismaticBarrier"] = 235450,
		["MassPolymorph"] = 383121,
		["Foresight"] = 384861,
		["ArcaneTempo"] = 383980,
		["ConcentratedPower"] = 414379,
		["ImprovedArcaneMissiles"] = 383661,
		["Reduplication"] = 382569,
		["ImprovedFrostNova"] = 343183,
		["TouchoftheMagi"] = 321507,
		["ConeofCold"] = 120,
		["PresenceofMind"] = 205025,
		["CracklingEnergy"] = 321752,
		["Frostbolt"] = 116,
		["MassInvisibility"] = 414664,
		["Slipstream"] = 236457,
		["ManaAdept"] = 321526,
		["Impetus"] = 383676,
		["AccumulativeShielding"] = 382800,
		["HarmonicEcho"] = 384683,
		["ImprovedClearcasting"] = 321420,
		["TimeWarp"] = 80353,
		["AlterTime"] = 342245,
		["Resonance"] = 205028,
		["ImprovedPrismaticBarrier"] = 321745,
		["SiphonStorm"] = 384187,
		["RadiantSpark"] = 376103,
		["ArcaneOrb"] = 153626,
		["MasterySavant"] = 190740,
		["GreaterInvisibility"] = 110959,
		["ArcaneExplosion"] = 1449,
		["ArcaneIntellect"] = 1459,
		["ArcaneBombardment"] = 384581,
		["Counterspell"] = 2139,
		["FreezingCold"] = 386763,
		["Shimmer"] = 212653,
		["MassSlow"] = 391102,
		["Evocation"] = 12051,
		["ConjureRefreshment"] = 190336,
		["Supernova"] = 157980,
		},
		},
		["Demon Hunter"] = {
		["Havoc"] = {
		["Inertia"] = 427640,
		["ElysianDecree"] = 390163,
		["CriticalChaos"] = 320413,
		["FelBarrage"] = 258925,
		["EssenceBreak"] = 258860,
		["PitchBlack"] = 389783,
		["TrailofRuin"] = 258881,
		["BurningWound"] = 391189,
		["Pursuit"] = 320654,
		["LooksCanKill"] = 320415,
		["UnboundChaos"] = 347461,
		["GrowingInferno"] = 390158,
		["ShatteredSouls"] = 178940,
		["FelEruption"] = 211881,
		["SigilofFlame"] = 204596,
		["CharredWarblades"] = 213010,
		["ChaosStrike"] = 344862,
		["Blur"] = 198589,
		["MortalDance"] = 328725,
		["AuraofPain"] = 207347,
		["DemonHide"] = 428241,
		["DemonicAppetite"] = 206478,
		["DemonMuzzle"] = 388111,
		["Torment"] = 185245,
		["BlazingPath"] = 320416,
		["DemonsBite"] = 344859,
		["Darkness"] = 196718,
		["Disrupt"] = 183752,
		["ImprovedDisrupt"] = 320361,
		["SoulSigils"] = 395446,
		["ImmolationAura"] = 258920,
		["SerratedGlaive"] = 390154,
		["SwallowedAnger"] = 320313,
		["SpectralSight"] = 188501,
		["UnrestrainedFury"] = 320770,
		["Initiative"] = 388108,
		["BurningHatred"] = 320374,
		["ScarsofSuffering"] = 428232,
		["VengefulBonds"] = 320635,
		["InternalStruggle"] = 393822,
		["ConsumeMagic"] = 278326,
		["AnyMeansNecessary"] = 388114,
		["BlindFury"] = 203550,
		["InnerDemon"] = 389693,
		["SigilofMisery"] = 207684,
		["RestlessHunter"] = 390142,
		["ChampionoftheGlaive"] = 429211,
		["SoulRending"] = 204909,
		["ThrowGlaive"] = 185123,
		["FirstBlood"] = 206416,
		["LivebytheGlaive"] = 428607,
		["Netherwalk"] = 196555,
		["DancingwithFate"] = 389978,
		["Imprison"] = 217832,
		["FuriousGaze"] = 343311,
		["DesperateInstincts"] = 205411,
		["RelentlessOnslaught"] = 389977,
		["FelRush"] = 344865,
		["FuriousThrows"] = 393029,
		["CycleofHatred"] = 258887,
		["Ragefire"] = 388107,
		["InsatiableHunger"] = 258876,
		["AldrachiDesign"] = 391409,
		["EyeBeam"] = 198013,
		["TacticalRetreat"] = 389688,
		["BladeDance"] = 188499,
		["IsolatedPrey"] = 388113,
		["LongNight"] = 389781,
		["IllidariKnowledge"] = 389696,
		["ShatteredDestiny"] = 388116,
		["Glide"] = 131347,
		["Soulscar"] = 388106,
		["QuickenedSigils"] = 209281,
		["ChaosTheory"] = 389687,
		["RushofChaos"] = 320421,
		["Momentum"] = 206476,
		["ImprovedChaosStrike"] = 343206,
		["MasteryDemonicPresence"] = 185164,
		["ChaoticDisposition"] = 428492,
		["DisruptingFury"] = 183782,
		["LostinDarkness"] = 389849,
		["TheHunt"] = 370965,
		["AFireInside"] = 427775,
		["DemonicWards"] = 278386,
		["KnowYourEnemy"] = 388118,
		["ChaoticTransformation"] = 388112,
		["MasteroftheGlaive"] = 389763,
		["DashofChaos"] = 427794,
		["Metamorphosis"] = 191427,
		["CollectiveAnguish"] = 390152,
		["ChaosFragments"] = 320412,
		["PreciseSigils"] = 389799,
		["FelfireHaste"] = 389846,
		["FlamesofFury"] = 389694,
		["WilloftheIllidari"] = 389695,
		["ImprovedSigilofMisery"] = 320418,
		["GlaiveTempest"] = 342817,
		["ChaosNova"] = 179057,
		["Demonic"] = 213410,
		["ShatteredRestoration"] = 389824,
		["Felblade"] = 232893,
		["InfernalArmor"] = 320331,
		["ImprovedFelRush"] = 343017,
		["ChaosBrand"] = 255260,
		["VengefulRetreat"] = 198793,
		["AcceleratedBlade"] = 391275,
		["ErraticFelheart"] = 391397,
		["DemonBlades"] = 203555,
		["BouncingGlaives"] = 320386,
		["DoubleJump"] = 196055,
		["DeflectingDance"] = 427776,
		},
		["Vengeance"] = {
		["SigilofFlame"] = 204596,
		["ElysianDecree"] = 390163,
		["InfernalStrike"] = 189110,
		["Torment"] = 185245,
		["IlluminatedSigils"] = 428557,
		["Fracture"] = 263642,
		["CharredWarblades"] = 213010,
		["Soulcrush"] = 389985,
		["FieryBrand"] = 204021,
		["Pursuit"] = 320654,
		["ChainsofAnger"] = 389715,
		["SoulCarver"] = 207407,
		["VolatileFlameblood"] = 390808,
		["FieryDemise"] = 389220,
		["FelDevastation"] = 212084,
		["StoketheFlames"] = 393827,
		["ShatteredSouls"] = 204254,
		["CycleofBinding"] = 389718,
		["AuraofPain"] = 207347,
		["VengefulBonds"] = 320635,
		["DemonMuzzle"] = 388111,
		["SpectralSight"] = 188501,
		["BlazingPath"] = 320416,
		["SoulRending"] = 204909,
		["BouncingGlaives"] = 320386,
		["DemonSpikes"] = 203720,
		["ImprovedDisrupt"] = 320361,
		["LastResort"] = 209258,
		["AscendingFlame"] = 428603,
		["ImmolationAura"] = 258920,
		["SigilofChains"] = 202138,
		["SigilofSilence"] = 202137,
		["PreciseSigils"] = 389799,
		["Disrupt"] = 183752,
		["DarkglareBoon"] = 389708,
		["InfernalArmor"] = 320331,
		["ThickSkin"] = 320380,
		["Painbringer"] = 207387,
		["ConsumeMagic"] = 278326,
		["FeedtheDemon"] = 218612,
		["Soulmonger"] = 389711,
		["PerfectlyBalancedGlaive"] = 320387,
		["SoulSigils"] = 395446,
		["DowninFlames"] = 389732,
		["ChampionoftheGlaive"] = 429211,
		["MeteoricStrikes"] = 389724,
		["ThrowGlaive"] = 185123,
		["DisruptingFury"] = 183782,
		["LivebytheGlaive"] = 428607,
		["MasteryFelBlood"] = 203747,
		["ShatteredRestoration"] = 389824,
		["Demonic"] = 213410,
		["ChaosNova"] = 179057,
		["SoulBarrier"] = 263648,
		["SwallowedAnger"] = 320313,
		["FelRush"] = 344865,
		["ImprovedSigilofMisery"] = 320418,
		["Imprison"] = 217832,
		["BurningBlood"] = 390213,
		["SoulFurnace"] = 391165,
		["BurningAlive"] = 207739,
		["TheHunt"] = 370965,
		["FocusedCleave"] = 343207,
		["AldrachiDesign"] = 391409,
		["SigilofMisery"] = 207684,
		["DeflectingSpikes"] = 321028,
		["IllidariKnowledge"] = 389696,
		["BulkExtraction"] = 320341,
		["ChaosFragments"] = 320412,
		["AgonizingFlames"] = 207548,
		["QuickenedSigils"] = 209281,
		["ExtendedSpikes"] = 389721,
		["RushofChaos"] = 320421,
		["CollectiveAnguish"] = 390152,
		["Darkness"] = 196718,
		["PitchBlack"] = 389783,
		["MasteroftheGlaive"] = 389763,
		["WilloftheIllidari"] = 389695,
		["LostinDarkness"] = 389849,
		["ErraticFelheart"] = 391397,
		["LongNight"] = 389781,
		["DemonicWards"] = 203513,
		["SpiritBomb"] = 247454,
		["Frailty"] = 389958,
		["Fallout"] = 227174,
		["SoulCleave"] = 228477,
		["Metamorphosis"] = 191427,
		["FelFlameFortification"] = 389705,
		["RoaringFire"] = 391178,
		["FeastofSouls"] = 207697,
		["Glide"] = 131347,
		["FlamesofFury"] = 389694,
		["Vulnerability"] = 389976,
		["RevelinPain"] = 343014,
		["CharredFlesh"] = 336639,
		["VengefulRetreat"] = 198793,
		["InternalStruggle"] = 393822,
		["FelfireHaste"] = 389846,
		["Felblade"] = 232893,
		["RuinousBulwark"] = 326853,
		["UnrestrainedFury"] = 320770,
		["ShearFury"] = 389997,
		["ChaosBrand"] = 255260,
		["CalcifiedSpikes"] = 389720,
		["Retaliation"] = 389729,
		["VoidReaver"] = 268175,
		["DemonsBite"] = 344859,
		["DoubleJump"] = 196055,
		["ChaosStrike"] = 344862,
		},
		},
		["Warlock"] = {
		["Demonology"] = {
		["Soulburn"] = 385899,
		["SargereiTechnique"] = 405955,
		["Shadowflame"] = 384069,
		["UnendingBreath"] = 5697,
		["GrimoireofSynergy"] = 171975,
		["Demoniac"] = 426115,
		["SoulLeech"] = 108370,
		["ImpGangBoss"] = 387445,
		["Horrify"] = 56244,
		["CarnivorousStalkers"] = 386194,
		["HeavyHanded"] = 416183,
		["RitualofDoom"] = 342601,
		["WrathfulMinion"] = 386864,
		["ReignofTyranny"] = 427684,
		["FelandSteel"] = 386200,
		["DemonicResilience"] = 389590,
		["SubjugateDemon"] = 1098,
		["BilescourgeBombers"] = 267211,
		["AccruedVitality"] = 386613,
		["CommandDemon"] = 119898,
		["FelInvocation"] = 428351,
		["NerzhulsVolition"] = 387526,
		["MortalCoil"] = 6789,
		["GrandWarlocksDesign"] = 387084,
		["CursesofEnfeeblement"] = 386105,
		["AmplifyCurse"] = 328774,
		["Darkfury"] = 264874,
		["ImmutableHatred"] = 405670,
		["Soulstone"] = 20707,
		["DemonSkin"] = 219272,
		["InnerDemons"] = 267216,
		["CurseofWeakness"] = 702,
		["KazaaksFinalCurse"] = 387483,
		["DemonicEmbrace"] = 288843,
		["SummonVilefiend"] = 264119,
		["Fear"] = 5782,
		["HowlofTerror"] = 5484,
		["Demonbolt"] = 264178,
		["SoulConduit"] = 215941,
		["DemonicStrength"] = 267171,
		["FelDomination"] = 333889,
		["UmbralBlaze"] = 405798,
		["Imperator"] = 416230,
		["Guillotine"] = 386833,
		["FelSynergy"] = 389367,
		["DemonicInspiration"] = 386858,
		["SummonDemonicTyrant"] = 265187,
		["Doom"] = 603,
		["AnnihilanTraining"] = 386174,
		["InfernalCommand"] = 387549,
		["ShadowBolt"] = 686,
		["StrengthofWill"] = 317138,
		["FrequentDonor"] = 386686,
		["Corruption"] = 172,
		["SoulStrike"] = 428344,
		["ShadowInvocation"] = 422054,
		["IchorofDevils"] = 386664,
		["DrainLife"] = 234153,
		["DemonicCore"] = 267102,
		["Implosion"] = 196277,
		["DemonicKnowledge"] = 386185,
		["SweetSouls"] = 386620,
		["FelPact"] = 386113,
		["StolenPower"] = 387602,
		["TheExpendables"] = 387600,
		["SoulboundTyrant"] = 334585,
		["FelSunder"] = 387399,
		["DemonicCircle"] = 48018,
		["CreateSoulwell"] = 29893,
		["CurseofExhaustion"] = 334275,
		["FiendishStride"] = 386110,
		["SocretharsGuile"] = 405936,
		["HandofGuldan"] = 105174,
		["GrimFeast"] = 386689,
		["TheHoundmastersStratagem"] = 267170,
		["ResoluteBarrier"] = 389359,
		["MasteryMasterDemonologist"] = 77219,
		["CreateHealthstone"] = 6201,
		["TeachingsoftheSatyr"] = 387972,
		["Dreadlash"] = 264078,
		["SacrificedSouls"] = 267214,
		["DemonicFortitude"] = 386617,
		["BurningRush"] = 111400,
		["SpellLock"] = 119910,
		["CurseofTongues"] = 1714,
		["TeachingsoftheBlackHarvest"] = 385881,
		["DarkPact"] = 108416,
		["DemonicCircleTeleport"] = 48020,
		["Nightmare"] = 386648,
		["DemonicCalling"] = 205145,
		["Shadowfury"] = 30283,
		["Banish"] = 710,
		["GrimoireFelguard"] = 111898,
		["Lifeblood"] = 386646,
		["InquisitorsGaze"] = 386344,
		["Cavitation"] = 416154,
		["ShadowsBite"] = 387322,
		["NetherPortal"] = 267217,
		["SoulLink"] = 108415,
		["GuldansAmbition"] = 387578,
		["MaleficImpact"] = 416341,
		["TheCodexofXerrath"] = 101508,
		["EyeofKilrogg"] = 126,
		["DemonicGateway"] = 111771,
		["CallDreadstalkers"] = 104316,
		["PowerSiphon"] = 264130,
		["HealthFunnel"] = 755,
		["SpitefulReconstitution"] = 428394,
		["AbyssWalker"] = 389609,
		["RitualofSummoning"] = 698,
		["PactoftheImpMother"] = 387541,
		["DarkAccord"] = 386659,
		["ProfaneBargain"] = 389576,
		["DreadCalling"] = 387391,
		["SummonSoulkeeper"] = 386256,
		["UnendingResolve"] = 104773,
		["AntoranArmaments"] = 387494,
		["FelArmor"] = 386124,
		["SoulShards"] = 246985,
		},
		["Destruction"] = {
		["Soulburn"] = 385899,
		["SargereiTechnique"] = 405955,
		["Shadowflame"] = 384069,
		["UnendingBreath"] = 5697,
		["GrimoireofSynergy"] = 171975,
		["Decimation"] = 387176,
		["Backdraft"] = 196406,
		["SoulLeech"] = 108370,
		["Horrify"] = 56244,
		["Inferno"] = 270545,
		["GrimoireofSacrifice"] = 108503,
		["SummonInfernal"] = 1122,
		["ScaldingFlames"] = 388832,
		["WrathfulMinion"] = 386864,
		["SubjugateDemon"] = 1098,
		["Shadowburn"] = 17877,
		["AccruedVitality"] = 386613,
		["SoulShards"] = 246985,
		["Immolate"] = 348,
		["DemonicResilience"] = 389590,
		["MortalCoil"] = 6789,
		["GrandWarlocksDesign"] = 387084,
		["TheCodexofXerrath"] = 101508,
		["AmplifyCurse"] = 328774,
		["CursesofEnfeeblement"] = 386105,
		["ImprovedConflagrate"] = 231793,
		["UnendingResolve"] = 104773,
		["DemonSkin"] = 219272,
		["RollingHavoc"] = 387569,
		["Soulstone"] = 20707,
		["MasteryChaoticEnergies"] = 77220,
		["DemonicEmbrace"] = 288843,
		["FireandBrimstone"] = 196408,
		["Backlash"] = 387384,
		["HowlofTerror"] = 5484,
		["EyeofKilrogg"] = 126,
		["SoulConduit"] = 215941,
		["DiabolicEmbers"] = 387173,
		["FelDomination"] = 333889,
		["ReverseEntropy"] = 205148,
		["AvatarofDestruction"] = 387159,
		["PowerOverwhelming"] = 387279,
		["FelSynergy"] = 389367,
		["DemonicInspiration"] = 386858,
		["Chaosbringer"] = 422057,
		["CurseofWeakness"] = 702,
		["RitualofDoom"] = 342601,
		["Havoc"] = 80240,
		["FrequentDonor"] = 386686,
		["StrengthofWill"] = 317138,
		["ShadowBolt"] = 686,
		["BurntoAshes"] = 387153,
		["RainofFire"] = 5740,
		["Mayhem"] = 387506,
		["ChaosBolt"] = 116858,
		["DrainLife"] = 234153,
		["InfernalBrand"] = 387475,
		["RoaringBlaze"] = 205184,
		["IchorofDevils"] = 386664,
		["SweetSouls"] = 386620,
		["FelPact"] = 386113,
		["Fear"] = 5782,
		["ImprovedImmolate"] = 387093,
		["InquisitorsGaze"] = 386344,
		["CurseofExhaustion"] = 334275,
		["DemonicCircle"] = 48018,
		["CreateSoulwell"] = 29893,
		["Pandemonium"] = 387509,
		["FiendishStride"] = 386110,
		["RitualofRuin"] = 387156,
		["BurningRush"] = 111400,
		["GrimFeast"] = 386689,
		["CreateHealthstone"] = 6201,
		["SocretharsGuile"] = 405936,
		["Banish"] = 710,
		["ChannelDemonfire"] = 196447,
		["TeachingsoftheSatyr"] = 387972,
		["Ruin"] = 387103,
		["Eradication"] = 196412,
		["DemonicFortitude"] = 386617,
		["MasterRitualist"] = 387165,
		["Shadowfury"] = 30283,
		["CurseofTongues"] = 1714,
		["CryHavoc"] = 387522,
		["DarkPact"] = 108416,
		["DemonicCircleTeleport"] = 48020,
		["Nightmare"] = 386648,
		["InternalCombustion"] = 266134,
		["Flashpoint"] = 387259,
		["CrashingChaos"] = 417234,
		["ChaosIncarnate"] = 387275,
		["Lifeblood"] = 386646,
		["Conflagrate"] = 17962,
		["ExplosivePotential"] = 388827,
		["ConflagrationofChaos"] = 387108,
		["ResoluteBarrier"] = 389359,
		["SoulFire"] = 6353,
		["TeachingsoftheBlackHarvest"] = 385881,
		["Pyrogenics"] = 387095,
		["RagingDemonfire"] = 387166,
		["DimensionalRift"] = 387976,
		["DemonicGateway"] = 111771,
		["Cataclysm"] = 152108,
		["Darkfury"] = 264874,
		["HealthFunnel"] = 755,
		["Incinerate"] = 29722,
		["AbyssWalker"] = 389609,
		["RitualofSummoning"] = 698,
		["SoulLink"] = 108415,
		["DarkAccord"] = 386659,
		["RainofChaos"] = 266086,
		["ProfaneBargain"] = 389576,
		["SummonSoulkeeper"] = 386256,
		["AshenRemains"] = 387252,
		["CommandDemon"] = 119898,
		["FelArmor"] = 386124,
		["Corruption"] = 172,
		},
		["Affliction"] = {
		["Soulburn"] = 385899,
		["SargereiTechnique"] = 405955,
		["Shadowflame"] = 384069,
		["UnendingBreath"] = 5697,
		["GrimoireofSynergy"] = 171975,
		["PandemicInvocation"] = 386759,
		["KindledMalice"] = 405330,
		["SoulFlame"] = 199471,
		["Horrify"] = 56244,
		["RitualofDoom"] = 342601,
		["SacrolashsDarkStrike"] = 386986,
		["AbsoluteCorruption"] = 196103,
		["GrimoireofSacrifice"] = 108503,
		["WrathfulMinion"] = 386864,
		["DoomBlossom"] = 416621,
		["MasteryPotentAfflictions"] = 77215,
		["SubjugateDemon"] = 1098,
		["AccruedVitality"] = 386613,
		["DemonicResilience"] = 389590,
		["MortalCoil"] = 6789,
		["GrandWarlocksDesign"] = 387084,
		["PhantomSingularity"] = 205179,
		["AmplifyCurse"] = 328774,
		["CursesofEnfeeblement"] = 386105,
		["WitheringBolt"] = 386976,
		["DemonSkin"] = 219272,
		["ProfaneBargain"] = 389576,
		["Agony"] = 980,
		["HowlofTerror"] = 5484,
		["SeizedVitality"] = 387250,
		["SoulConduit"] = 215941,
		["TheCodexofXerrath"] = 101508,
		["FelDomination"] = 333889,
		["SoulShards"] = 246985,
		["UnendingResolve"] = 104773,
		["SoulLeech"] = 108370,
		["FelSynergy"] = 389367,
		["DemonicInspiration"] = 386858,
		["DemonicEmbrace"] = 288843,
		["Soulstone"] = 20707,
		["SoulLink"] = 108415,
		["EyeofKilrogg"] = 126,
		["Corruption"] = 172,
		["StrengthofWill"] = 317138,
		["FrequentDonor"] = 386686,
		["ShadowBolt"] = 686,
		["Nightfall"] = 108558,
		["DarkHarvest"] = 387016,
		["DrainSoul"] = 198590,
		["DrainLife"] = 234153,
		["HealthFunnel"] = 755,
		["DarkVirtuosity"] = 405327,
		["IchorofDevils"] = 386664,
		["SweetSouls"] = 386620,
		["FelPact"] = 386113,
		["InevitableDemise"] = 334319,
		["WritheinAgony"] = 196102,
		["GrimFeast"] = 386689,
		["CreepingDeath"] = 264000,
		["Haunt"] = 48181,
		["CreateSoulwell"] = 29893,
		["XavianTeachings"] = 317031,
		["FiendishStride"] = 386110,
		["ShadowEmbrace"] = 32388,
		["CreateHealthstone"] = 6201,
		["CurseofExhaustion"] = 334275,
		["SpellLock"] = 119910,
		["ResoluteBarrier"] = 389359,
		["Banish"] = 710,
		["SoulRot"] = 386997,
		["TeachingsoftheSatyr"] = 387972,
		["BurningRush"] = 111400,
		["DemonicCircleTeleport"] = 48020,
		["DemonicFortitude"] = 386617,
		["VileTaint"] = 278350,
		["WrathofConsumption"] = 387065,
		["CurseofTongues"] = 1714,
		["GrimReach"] = 389992,
		["DarkPact"] = 108416,
		["FocusedMalignancy"] = 399668,
		["Nightmare"] = 386648,
		["SeedofCorruption"] = 27243,
		["MaleficRapture"] = 324536,
		["XaviusGambit"] = 416615,
		["UnstableAffliction"] = 316099,
		["Lifeblood"] = 386646,
		["SiphonLife"] = 63106,
		["SowtheSeeds"] = 196226,
		["TeachingsoftheBlackHarvest"] = 385881,
		["MalevolentVisionary"] = 387273,
		["SoulSwap"] = 386951,
		["SoulEatersGluttony"] = 389630,
		["AgonizingCorruption"] = 386922,
		["HauntedSoul"] = 387301,
		["SocretharsGuile"] = 405936,
		["DemonicGateway"] = 111771,
		["Shadowfury"] = 30283,
		["DemonicCircle"] = 48018,
		["TormentedCrescendo"] = 387075,
		["InquisitorsGaze"] = 386344,
		["AbyssWalker"] = 389609,
		["RitualofSummoning"] = 698,
		["CurseofWeakness"] = 702,
		["DarkAccord"] = 386659,
		["Fear"] = 5782,
		["Darkfury"] = 264874,
		["SummonSoulkeeper"] = 386256,
		["DreadTouch"] = 389775,
		["CommandDemon"] = 119898,
		["FelArmor"] = 386124,
		["SummonDarkglare"] = 205180,
		},
		},
		["Death Knight"] = {
		["Unholy"] = {
		["MightofThassarian"] = 374111,
		["UnholyCommand"] = 316941,
		["ImprovedFesteringStrike"] = 316867,
		["UnholyAssault"] = 207289,
		["Assimilation"] = 374383,
		["AntiMagicZone"] = 51052,
		["DeathsReach"] = 276079,
		["ScourgeStrike"] = 55090,
		["UnholyBond"] = 374261,
		["MindFreeze"] = 47528,
		["RuneMastery"] = 374574,
		["RupturedViscera"] = 390236,
		["WilloftheNecropolis"] = 206967,
		["SoulReaper"] = 343294,
		["ChainsofIce"] = 45524,
		["AbominationLimb"] = 383269,
		["ImprovedDeathCoil"] = 377580,
		["MarchofDarkness"] = 391546,
		["DarkCommand"] = 56222,
		["Epidemic"] = 207317,
		["AntiMagicShell"] = 48707,
		["ClawingShadows"] = 207311,
		["UnholyEndurance"] = 389682,
		["CommanderoftheDead"] = 390259,
		["InfectedClaws"] = 207272,
		["ControlUndead"] = 111673,
		["ProliferatingChill"] = 373930,
		["WraithWalk"] = 212552,
		["RunicCorruption"] = 51462,
		["MasteryDreadblade"] = 77515,
		["DeathsAdvance"] = 48265,
		["DarkTransformation"] = 63560,
		["GloomWard"] = 391571,
		["IceboundFortitude"] = 48792,
		["FeastingStrikes"] = 390161,
		["HarbingerofDoom"] = 276023,
		["Acclimation"] = 373926,
		["Morbidity"] = 377592,
		["Outbreak"] = 77575,
		["EternalAgony"] = 390268,
		["MercilessStrikes"] = 373923,
		["InsidiousChill"] = 391566,
		["UnholyGround"] = 374265,
		["DeathGate"] = 50977,
		["ReplenishingWounds"] = 377585,
		["EmpowerRuneWeapon"] = 47568,
		["Reaping"] = 377514,
		["DeathsEcho"] = 356367,
		["Plaguebringer"] = 390175,
		["BlindingSleet"] = 207167,
		["VileContagion"] = 390279,
		["Lichborne"] = 49039,
		["DeathCoil"] = 47541,
		["DeathRot"] = 377537,
		["SuddenDoom"] = 49530,
		["ArmyoftheDamned"] = 276837,
		["ArmyoftheDead"] = 42650,
		["GripoftheDead"] = 273952,
		["Festermight"] = 377590,
		["FesteringStrike"] = 85948,
		["DeathGrip"] = 49576,
		["RunicMastery"] = 390166,
		["DeathStrike"] = 49998,
		["ClenchingGrasp"] = 389679,
		["OnaPaleHorse"] = 51986,
		["DeathPact"] = 48743,
		["Superstrain"] = 390283,
		["RaiseAlly"] = 61999,
		["DarkSuccor"] = 178819,
		["SummonGargoyle"] = 49206,
		["Enfeeble"] = 392566,
		["Coldthirst"] = 378848,
		["GhoulishFrenzy"] = 377587,
		["RunicAttenuation"] = 207104,
		["PathofFrost"] = 3714,
		["UnholyPact"] = 319230,
		["Pestilence"] = 277234,
		["Runeforging"] = 53428,
		["BloodDraw"] = 374598,
		["Defile"] = 152280,
		["RuneStrike"] = 316239,
		["RaiseDead"] = 46585,
		["Suppression"] = 374049,
		["VeteranoftheThirdWar"] = 48263,
		["AllWillServe"] = 194916,
		["UnholyBlight"] = 115989,
		["UnholyAura"] = 377440,
		["CleavingStrikes"] = 316916,
		["ImprovedDeathStrike"] = 374277,
		["RottenTouch"] = 390275,
		["EbonFever"] = 207269,
		["Permafrost"] = 207200,
		["BloodScent"] = 374030,
		["DeathandDecay"] = 43265,
		["CoilofDevastation"] = 390270,
		["MagusoftheDead"] = 390196,
		["AntiMagicBarrier"] = 205727,
		["SacrificialPact"] = 327574,
		["Brittle"] = 374504,
		["Asphyxiate"] = 221562,
		["IcyTalons"] = 194878,
		["BurstingSores"] = 207264,
		["Apocalypse"] = 275699,
		},
		["Frost"] = {
		["MightofThassarian"] = 374111,
		["ImprovedFrostStrike"] = 316803,
		["Assimilation"] = 374383,
		["InvigoratingFreeze"] = 377092,
		["AntiMagicZone"] = 51052,
		["DeathsReach"] = 276079,
		["RunicCommand"] = 376251,
		["FrostwhelpsAid"] = 377226,
		["UnholyBond"] = 374261,
		["ColdBloodedRage"] = 377083,
		["ShatteringBlade"] = 207057,
		["MindFreeze"] = 47528,
		["RuneMastery"] = 374574,
		["DeathsAdvance"] = 48265,
		["WilloftheNecropolis"] = 206967,
		["SoulReaper"] = 343294,
		["ChainsofIce"] = 45524,
		["SacrificialPact"] = 327574,
		["FrostwyrmsFury"] = 279302,
		["MarchofDarkness"] = 391546,
		["ImprovedRime"] = 316838,
		["FrigidExecutioner"] = 377073,
		["AntiMagicShell"] = 48707,
		["MightoftheFrozenWastes"] = 81333,
		["UnholyEndurance"] = 389682,
		["Everfrost"] = 376938,
		["UnleashedFrenzy"] = 376905,
		["ImprovedObliterate"] = 317198,
		["ControlUndead"] = 111673,
		["PiercingChill"] = 377351,
		["AbsoluteZero"] = 377047,
		["ProliferatingChill"] = 373930,
		["WraithWalk"] = 212552,
		["PillarofFrost"] = 51271,
		["MurderousEfficiency"] = 207061,
		["IceboundFortitude"] = 48792,
		["RunicEmpowerment"] = 81229,
		["MasteryFrozenHeart"] = 77514,
		["Acclimation"] = 373926,
		["MercilessStrikes"] = 373923,
		["FrostStrike"] = 49143,
		["CleavingStrikes"] = 316916,
		["DeathandDecay"] = 43265,
		["Brittle"] = 374504,
		["EmpowerRuneWeapon"] = 47568,
		["InexorableAssault"] = 253593,
		["InsidiousChill"] = 391566,
		["EnduringChill"] = 377376,
		["BlindingSleet"] = 207167,
		["DarkCommand"] = 56222,
		["AbominationLimb"] = 383269,
		["KillingMachine"] = 51128,
		["DeathCoil"] = 47541,
		["DeathsEcho"] = 356367,
		["HornofWinter"] = 57330,
		["Avalanche"] = 207142,
		["GripoftheDead"] = 273952,
		["BitingCold"] = 377056,
		["DeathGate"] = 50977,
		["DeathGrip"] = 49576,
		["Obliterate"] = 49020,
		["DeathStrike"] = 49998,
		["ColdHeart"] = 281208,
		["HowlingBlast"] = 49184,
		["DeathPact"] = 48743,
		["DarkSuccor"] = 178819,
		["Lichborne"] = 49039,
		["OnaPaleHorse"] = 51986,
		["GatheringStorm"] = 194912,
		["Enfeeble"] = 392566,
		["UnholyGround"] = 374265,
		["GlacialAdvance"] = 194913,
		["VeteranoftheThirdWar"] = 48263,
		["PathofFrost"] = 3714,
		["RaiseAlly"] = 61999,
		["Runeforging"] = 53428,
		["RunicAttenuation"] = 207104,
		["Icecap"] = 207126,
		["BloodDraw"] = 374598,
		["RuneStrike"] = 316239,
		["EnduringStrength"] = 377190,
		["Suppression"] = 374049,
		["Icebreaker"] = 392950,
		["RaiseDead"] = 46585,
		["ClenchingGrasp"] = 389679,
		["BreathofSindragosa"] = 152279,
		["ChillStreak"] = 305392,
		["ImprovedDeathStrike"] = 374277,
		["Obliteration"] = 281238,
		["Frostscythe"] = 207230,
		["Permafrost"] = 207200,
		["BloodScent"] = 374030,
		["RemorselessWinter"] = 196770,
		["Rime"] = 59057,
		["IcyTalons"] = 194878,
		["AntiMagicBarrier"] = 205727,
		["RageoftheFrozenChampion"] = 377076,
		["GloomWard"] = 391571,
		["Asphyxiate"] = 221562,
		["Coldthirst"] = 378848,
		["Bonegrinder"] = 377098,
		["FatalFixation"] = 405166,
		},
		["Blood"] = {
		["MightofThassarian"] = 374111,
		["DeathsCaress"] = 195292,
		["Ossuary"] = 219786,
		["Assimilation"] = 374383,
		["BloodBoil"] = 50842,
		["BloodTap"] = 221699,
		["AntiMagicZone"] = 51052,
		["DeathsReach"] = 276079,
		["UnholyBond"] = 374261,
		["MindFreeze"] = 47528,
		["RuneMastery"] = 374574,
		["DeathsAdvance"] = 48265,
		["WilloftheNecropolis"] = 206967,
		["UmbilicusEternus"] = 391517,
		["ChainsofIce"] = 45524,
		["SacrificialPact"] = 327574,
		["MarchofDarkness"] = 391546,
		["DarkCommand"] = 56222,
		["AntiMagicShell"] = 48707,
		["Marrowrend"] = 195182,
		["UnholyEndurance"] = 389682,
		["Bonestorm"] = 194844,
		["BloodFeast"] = 391386,
		["ControlUndead"] = 111673,
		["BloodFortification"] = 374721,
		["ProliferatingChill"] = 373930,
		["WraithWalk"] = 212552,
		["Coldthirst"] = 378848,
		["ImprovedHeartStrike"] = 374717,
		["IceboundFortitude"] = 48792,
		["Tombstone"] = 219809,
		["CrimsonScourge"] = 81136,
		["VampiricBlood"] = 55233,
		["Blooddrinker"] = 206931,
		["Brittle"] = 374504,
		["RelishinBlood"] = 317610,
		["SoulReaper"] = 343294,
		["Acclimation"] = 373926,
		["CleavingStrikes"] = 316916,
		["MercilessStrikes"] = 373923,
		["DeathandDecay"] = 43265,
		["DancingRuneWeapon"] = 49028,
		["InsidiousChill"] = 391566,
		["DeathGate"] = 50977,
		["EmpowerRuneWeapon"] = 47568,
		["LeechingStrike"] = 377629,
		["DeathsEcho"] = 356367,
		["FoulBulwark"] = 206974,
		["BlindingSleet"] = 207167,
		["HeartStrike"] = 206930,
		["ImprovedVampiricBlood"] = 317133,
		["Lichborne"] = 49039,
		["DeathCoil"] = 47541,
		["InsatiableBlade"] = 377637,
		["Hemostasis"] = 273946,
		["TighteningGrasp"] = 206970,
		["GripoftheDead"] = 273952,
		["PerseveranceoftheEbonBlade"] = 374747,
		["Heartbreaker"] = 221536,
		["DeathGrip"] = 49576,
		["ReinforcedBones"] = 374737,
		["DeathStrike"] = 49998,
		["EverlastingBond"] = 377668,
		["RaiseAlly"] = 61999,
		["DeathPact"] = 48743,
		["Bloodworms"] = 195679,
		["BloodDraw"] = 374598,
		["OnaPaleHorse"] = 51986,
		["Consumption"] = 274156,
		["Enfeeble"] = 392566,
		["RapidDecomposition"] = 194662,
		["Runeforging"] = 53428,
		["Coagulopathy"] = 391477,
		["PathofFrost"] = 3714,
		["SanguineGround"] = 391458,
		["RaiseDead"] = 46585,
		["RunicAttenuation"] = 207104,
		["RedThirst"] = 205723,
		["RuneTap"] = 194679,
		["RuneStrike"] = 316239,
		["Heartrend"] = 377655,
		["Suppression"] = 374049,
		["VeteranoftheThirdWar"] = 48263,
		["MasteryBloodShield"] = 77513,
		["ClenchingGrasp"] = 389679,
		["IronHeart"] = 391395,
		["Voracious"] = 273953,
		["ImprovedDeathStrike"] = 374277,
		["Purgatory"] = 114556,
		["Bloodshot"] = 391398,
		["Permafrost"] = 207200,
		["BloodScent"] = 374030,
		["UnholyGround"] = 374265,
		["GorefiendsGrasp"] = 108199,
		["MarkofBlood"] = 206940,
		["AntiMagicBarrier"] = 205727,
		["IcyTalons"] = 194878,
		["GloomWard"] = 391571,
		["Asphyxiate"] = 221562,
		["ImprovedBoneShield"] = 374715,
		["AbominationLimb"] = 383269,
		["ShatteringBone"] = 377640,
		},
		},
		["Priest"] = {
		["Shadow"] = {
		["Renew"] = 139,
		["VoidTorrent"] = 263165,
		["DistortedReality"] = 409044,
		["MentalDecay"] = 375994,
		["DarkEvangelism"] = 391095,
		["MindControl"] = 605,
		["PowerWordShield"] = 17,
		["IdolofYoggSaron"] = 373273,
		["ThroesofPain"] = 377422,
		["Mindgames"] = 375901,
		["TwistofFate"] = 390972,
		["MindSpike"] = 73510,
		["TormentedSpirits"] = 391284,
		["SurgeofInsanity"] = 391399,
		["Voidtouched"] = 407430,
		["MindsEye"] = 407470,
		["ProtectiveLight"] = 193063,
		["AuspiciousSpirits"] = 155271,
		["Hallucinations"] = 280752,
		["Shadowfiend"] = 34433,
		["Resurrection"] = 2006,
		["ShadowyApparitions"] = 341491,
		["Manipulation"] = 390996,
		["Rhapsody"] = 390622,
		["MentalFortitude"] = 377065,
		["EssenceDevourer"] = 415479,
		["Benevolence"] = 415416,
		["PhantasmalPathogen"] = 407469,
		["PsychicLink"] = 199484,
		["Mindbender"] = 200174,
		["BindingHeals"] = 368275,
		["Phantasm"] = 108942,
		["MassDispel"] = 32375,
		["ImprovedFade"] = 390670,
		["MaddeningTouch"] = 391228,
		["VampiricEmbrace"] = 15286,
		["Shadowform"] = 232698,
		["Deathspeaker"] = 392507,
		["Voidform"] = 228264,
		["MindSoothe"] = 453,
		["SanguineTeachings"] = 373218,
		["ImprovedFlashHeal"] = 393870,
		["MindVision"] = 2096,
		["AngelicBulwark"] = 108945,
		["VoidShield"] = 280749,
		["SheerTerror"] = 390919,
		["MasteryShadowWeaving"] = 343690,
		["MindDevourer"] = 373202,
		["ShackleUndead"] = 9484,
		["VoidShift"] = 108968,
		["FromDarknessComesLight"] = 390615,
		["BlessedRecovery"] = 390767,
		["InescapableTorment"] = 373427,
		["TranslucentImage"] = 373446,
		["Malediction"] = 373221,
		["ShatteredPerceptions"] = 391112,
		["Misery"] = 238558,
		["PurifyDisease"] = 213634,
		["PrayerofMending"] = 33076,
		["MentalAgility"] = 341167,
		["IdolofCThun"] = 377349,
		["ShadowyInsight"] = 375888,
		["TwinsoftheSunPriestess"] = 373466,
		["DeathandMadness"] = 321291,
		["ShadowWordPain"] = 589,
		["Fade"] = 586,
		["Sanlayn"] = 199855,
		["CrystallineReflection"] = 373457,
		["DevouringPlague"] = 335467,
		["ShadowCrash"] = 205385,
		["DivineStar"] = 122121,
		["PowerInfusion"] = 10060,
		["FocusedWill"] = 45243,
		["InsidiousIre"] = 373212,
		["ShadowWordDeath"] = 32379,
		["DominateMind"] = 205364,
		["Levitate"] = 1706,
		["FlashHeal"] = 2061,
		["Halo"] = 120644,
		["VoidTendrils"] = 108920,
		["UnfurlingDarkness"] = 341273,
		["AngelicFeather"] = 121536,
		["Inspiration"] = 390676,
		["AngelsMercy"] = 238100,
		["DarkAscension"] = 391109,
		["Smite"] = 585,
		["HolyNova"] = 132157,
		["LastWord"] = 263716,
		["LightsInspiration"] = 373450,
		["PetrifyingScream"] = 55676,
		["WhisperingShadows"] = 406777,
		["VoidBolt"] = 228266,
		["LeapofFaith"] = 73325,
		["PowerWordFortitude"] = 21562,
		["IdolofNZoth"] = 373280,
		["PowerWordLife"] = 373481,
		["MindMelt"] = 391090,
		["IdolofYShaarj"] = 373310,
		["Intangibility"] = 288733,
		["VoidEruption"] = 228260,
		["ThoughtHarvester"] = 406788,
		["UnwaveringWill"] = 373456,
		["VampiricTouch"] = 34914,
		["Apathy"] = 390668,
		["Silence"] = 15487,
		["Dispersion"] = 47585,
		["DispelMagic"] = 528,
		["MindBlast"] = 8092,
		["PsychicScream"] = 8122,
		["DesperatePrayer"] = 19236,
		["FocusedMending"] = 372354,
		["WordsofthePious"] = 377438,
		["PsychicHorror"] = 64044,
		["MovewithGrace"] = 390620,
		["AncientMadness"] = 341240,
		["Mastermind"] = 391151,
		["ScreamsoftheVoid"] = 375767,
		["SurgeofLight"] = 109186,
		["BodyandSoul"] = 64129,
		["PsychicVoice"] = 196704,
		["TitheEvasion"] = 373223,
		["SpellWarding"] = 390667,
		},
		["Discipline"] = {
		["Renew"] = 139,
		["PrayerofMending"] = 33076,
		["DarkIndulgence"] = 372972,
		["EnduringLuminescence"] = 390685,
		["VoidTendrils"] = 108920,
		["PainandSuffering"] = 390689,
		["MindControl"] = 605,
		["PowerWordShield"] = 17,
		["Exaltation"] = 373042,
		["WordsofthePious"] = 377438,
		["AbyssalReverie"] = 373054,
		["Mindgames"] = 375901,
		["LightsPromise"] = 322115,
		["InescapableTorment"] = 373427,
		["PowerWordRadiance"] = 194509,
		["ShadowCovenant"] = 314867,
		["Shadowfiend"] = 34433,
		["ProtectoroftheFrail"] = 373035,
		["ProtectiveLight"] = 193063,
		["LuminousBarrier"] = 271466,
		["Rhapsody"] = 390622,
		["DivineAegis"] = 47515,
		["DispelMagic"] = 528,
		["AegisofWrath"] = 238135,
		["Manipulation"] = 390996,
		["OverloadedwithLight"] = 421557,
		["BindingHeals"] = 368275,
		["EssenceDevourer"] = 415479,
		["VoidSummoner"] = 390770,
		["Resurrection"] = 2006,
		["Schism"] = 424509,
		["Mindbender"] = 123040,
		["DesperatePrayer"] = 19236,
		["MassResurrection"] = 212036,
		["BlazeofLight"] = 215768,
		["ImprovedFade"] = 390670,
		["MassDispel"] = 32375,
		["VampiricEmbrace"] = 15286,
		["SanguineTeachings"] = 373218,
		["ShadowWordDeath"] = 32379,
		["UltimatePenitence"] = 421453,
		["MindSoothe"] = 453,
		["PowerWordBarrier"] = 62618,
		["PainSuppression"] = 33206,
		["MindVision"] = 2096,
		["MasteryGrace"] = 271534,
		["VoidShield"] = 280749,
		["SheerTerror"] = 390919,
		["SinsoftheMany"] = 280391,
		["Castigation"] = 193134,
		["PoweroftheDarkSide"] = 198068,
		["VoidShift"] = 108968,
		["ImprovedPurify"] = 390632,
		["BlessedRecovery"] = 390767,
		["MindBlast"] = 8092,
		["Lenience"] = 238063,
		["TwistofFate"] = 390972,
		["ShatteredPerceptions"] = 391112,
		["Atonement"] = 81749,
		["PurifyDisease"] = 440006,
		["Evangelism"] = 246287,
		["MentalAgility"] = 341167,
		["TwinsoftheSunPriestess"] = 373466,
		["Rapture"] = 47536,
		["MaliciousIntent"] = 372969,
		["TranslucentImage"] = 373446,
		["BrightPupil"] = 390684,
		["Fade"] = 586,
		["Sanlayn"] = 199855,
		["CrystallineReflection"] = 373457,
		["BorrowedTime"] = 390691,
		["ShadowWordPain"] = 589,
		["DivineStar"] = 110744,
		["PowerInfusion"] = 10060,
		["HarshDiscipline"] = 373180,
		["FocusedWill"] = 45243,
		["Levitate"] = 1706,
		["WealandWoe"] = 390786,
		["Purify"] = 527,
		["FlashHeal"] = 2061,
		["Halo"] = 120517,
		["ThroesofPain"] = 377422,
		["ShackleUndead"] = 9484,
		["ShieldDiscipline"] = 197045,
		["Sanctuary"] = 231682,
		["AngelsMercy"] = 238100,
		["Inspiration"] = 390676,
		["Smite"] = 585,
		["LeapofFaith"] = 73325,
		["Contrition"] = 197419,
		["LightsInspiration"] = 373450,
		["PainTransformation"] = 372991,
		["HolyNova"] = 132157,
		["PetrifyingScream"] = 55676,
		["PainfulPunishment"] = 390686,
		["PowerWordFortitude"] = 21562,
		["UnwaveringWill"] = 373456,
		["PowerWordLife"] = 373481,
		["Indemnity"] = 373049,
		["AngelicBulwark"] = 108945,
		["DominateMind"] = 205364,
		["AngelicFeather"] = 121536,
		["RevelinPurity"] = 373003,
		["PurgetheWicked"] = 204197,
		["TrainofThought"] = 390693,
		["Apathy"] = 390668,
		["FromDarknessComesLight"] = 390615,
		["TwilightEquilibrium"] = 390705,
		["DeathandMadness"] = 321291,
		["HeavensWrath"] = 421558,
		["PsychicScream"] = 8122,
		["FocusedMending"] = 372354,
		["ImprovedFlashHeal"] = 393870,
		["Expiation"] = 390832,
		["TwilightCorruption"] = 373065,
		["MovewithGrace"] = 390620,
		["Phantasm"] = 108942,
		["Benevolence"] = 415416,
		["Penance"] = 47540,
		["SurgeofLight"] = 109186,
		["BodyandSoul"] = 64129,
		["PsychicVoice"] = 196704,
		["TitheEvasion"] = 373223,
		["SpellWarding"] = 390667,
		},
		["Holy"] = {
		["Renew"] = 139,
		["PrayerofMending"] = 33076,
		["Pontifex"] = 390980,
		["GuardianAngel"] = 200209,
		["Heal"] = 2060,
		["DesperateTimes"] = 391381,
		["Shadowfiend"] = 34433,
		["Mindgames"] = 375901,
		["VoidTendrils"] = 108920,
		["Orison"] = 390947,
		["MindControl"] = 605,
		["PowerWordShield"] = 17,
		["Benediction"] = 193157,
		["WordsofthePious"] = 377438,
		["MasteryEchoofLight"] = 77485,
		["HolyMending"] = 391154,
		["Restitution"] = 391124,
		["RenewedFaith"] = 341997,
		["ThroesofPain"] = 377422,
		["HolyFire"] = 14914,
		["Manipulation"] = 390996,
		["Lightwell"] = 372835,
		["ProtectiveLight"] = 193063,
		["DivineWord"] = 372760,
		["DesperatePrayer"] = 19236,
		["BodyandSoul"] = 64129,
		["SanctifiedPrayers"] = 196489,
		["Censure"] = 200199,
		["AngelicBulwark"] = 108945,
		["DivineHymn"] = 64843,
		["MindSoothe"] = 453,
		["EssenceDevourer"] = 415479,
		["Benevolence"] = 415416,
		["DispelMagic"] = 528,
		["TwistofFate"] = 390972,
		["PrismaticEchoes"] = 390967,
		["BindingHeals"] = 368275,
		["SymbolofHope"] = 64901,
		["SanguineTeachings"] = 373218,
		["DivineImage"] = 392988,
		["PrayerCircle"] = 321377,
		["VampiricEmbrace"] = 15286,
		["SheerTerror"] = 390919,
		["ShadowWordDeath"] = 32379,
		["MassResurrection"] = 212036,
		["HolyWordSanctify"] = 34861,
		["Lightweaver"] = 390992,
		["ImprovedFlashHeal"] = 393870,
		["MindVision"] = 2096,
		["HealingChorus"] = 390881,
		["DeathandMadness"] = 321291,
		["SpiritofRedemption"] = 20711,
		["ImprovedPurify"] = 390632,
		["TranslucentImage"] = 373446,
		["Apotheosis"] = 200183,
		["VoidShift"] = 108968,
		["FromDarknessComesLight"] = 390615,
		["BlessedRecovery"] = 390767,
		["CrystallineReflection"] = 373457,
		["ShatteredPerceptions"] = 391112,
		["GalesofSong"] = 372370,
		["TrailofLight"] = 200128,
		["ResonantWords"] = 372309,
		["PurifyDisease"] = 440006,
		["GuardianSpirit"] = 47788,
		["EmpyrealBlaze"] = 372616,
		["Rhapsody"] = 390622,
		["MentalAgility"] = 341167,
		["TwinsoftheSunPriestess"] = 373466,
		["DominateMind"] = 205364,
		["PrayersoftheVirtuous"] = 390977,
		["Fade"] = 586,
		["Sanlayn"] = 199855,
		["ShadowWordPain"] = 589,
		["Purify"] = 527,
		["SayYourPrayers"] = 391186,
		["DivineStar"] = 110744,
		["LightoftheNaaru"] = 196985,
		["AnsweredPrayers"] = 391387,
		["FocusedWill"] = 45243,
		["HolyNova"] = 132157,
		["RevitalizingPrayers"] = 391208,
		["Levitate"] = 1706,
		["UnwaveringWill"] = 373456,
		["Halo"] = 120517,
		["LeapofFaith"] = 73325,
		["PowerInfusion"] = 10060,
		["ShackleUndead"] = 9484,
		["Inspiration"] = 390676,
		["AngelsMercy"] = 238100,
		["PrayerofHealing"] = 596,
		["Smite"] = 585,
		["Afterlife"] = 196707,
		["LightsInspiration"] = 373450,
		["HolyWordChastise"] = 88625,
		["BurningVehemence"] = 372307,
		["VoiceofHarmony"] = 390994,
		["CosmicRipple"] = 238136,
		["PetrifyingScream"] = 55676,
		["PowerWordFortitude"] = 21562,
		["Epiphany"] = 414553,
		["PowerWordLife"] = 373481,
		["FlashHeal"] = 2061,
		["MiracleWorker"] = 235587,
		["GuardiansoftheLight"] = 196437,
		["EverlastingLight"] = 391161,
		["MassDispel"] = 32375,
		["AngelicFeather"] = 121536,
		["CrisisManagement"] = 390954,
		["Apathy"] = 390668,
		["Enlightenment"] = 193155,
		["FocusedMending"] = 372354,
		["VoidShield"] = 280749,
		["MindBlast"] = 8092,
		["PsychicScream"] = 8122,
		["DivineService"] = 391233,
		["ImprovedFade"] = 390670,
		["Resurrection"] = 2006,
		["CircleofHealing"] = 204883,
		["MovewithGrace"] = 390620,
		["Phantasm"] = 108942,
		["HolyWordSalvation"] = 265202,
		["PrayerfulLitany"] = 391209,
		["SurgeofLight"] = 109186,
		["HolyWordSerenity"] = 2050,
		["PsychicVoice"] = 196704,
		["TitheEvasion"] = 373223,
		["SpellWarding"] = 390667,
		},
		},
		["Hunter"] = {
		["Marksmanship"] = {
		["PoisonInjection"] = 378014,
		["NaturesEndurance"] = 388042,
		["BindingShackles"] = 321468,
		["WildernessMedicine"] = 343242,
		["WailingArrow"] = 392060,
		["IntheRhythm"] = 407404,
		["WindrunnersBarrage"] = 389866,
		["AimedShot"] = 19434,
		["LegacyoftheWindrunners"] = 406425,
		["ImprovedKillShot"] = 343248,
		["ScareBeast"] = 1513,
		["MasterMarksman"] = 260309,
		["Volley"] = 260243,
		["SentinelsProtection"] = 388057,
		["TargetPractice"] = 321287,
		["ConcussiveShot"] = 5116,
		["TrickShots"] = 257621,
		["ArcaneShot"] = 185358,
		["CrackShot"] = 321293,
		["Barrage"] = 120360,
		["Streamline"] = 260367,
		["Readiness"] = 389865,
		["MasterySniperTraining"] = 193468,
		["MultiShot"] = 257620,
		["TacticalReload"] = 400472,
		["HuntersAvoidance"] = 384799,
		["TranquilizingShot"] = 19801,
		["Salvo"] = 400456,
		["Pathfinding"] = 378002,
		["WindrunnersGuidance"] = 378905,
		["SentinelsPerception"] = 388056,
		["ImprovedTraps"] = 343247,
		["Camouflage"] = 199483,
		["WingClip"] = 195645,
		["KillerInstinct"] = 273887,
		["SteadyShot"] = 56641,
		["Intimidation"] = 19577,
		["RazorFragments"] = 384790,
		["CallingtheShots"] = 260404,
		["Exhilaration"] = 109304,
		["PlayDead"] = 209997,
		["HuntersMark"] = 257284,
		["FreezingTrap"] = 187650,
		["Flare"] = 1543,
		["Fetch"] = 125050,
		["RoarofSacrifice"] = 53480,
		["QuickLoad"] = 378771,
		["HuntersKnowledge"] = 378766,
		["FocusedAim"] = 378767,
		["PreciseShots"] = 260240,
		["KillShot"] = 53351,
		["UnerringVision"] = 386878,
		["Entrapment"] = 393344,
		["EagleEye"] = 6197,
		["AspectoftheCheetah"] = 186257,
		["LoneSurvivor"] = 388039,
		["SerpentstalkersTrickery"] = 378888,
		["ImprovedSteadyShot"] = 321018,
		["AlphaPredator"] = 269737,
		["SerpentSting"] = 271788,
		["KeenEyesight"] = 378004,
		["KillerAccuracy"] = 378765,
		["CommandPet"] = 272651,
		["AspectoftheTurtle"] = 186265,
		["Bulletstorm"] = 389019,
		["Bullseye"] = 204089,
		["ImprovedTranquilizingShot"] = 343244,
		["LightAmmo"] = 378913,
		["TarTrap"] = 187698,
		["CarefulAim"] = 260228,
		["BornToBeWild"] = 266921,
		["ScatterShot"] = 213691,
		["SurvivaloftheFittest"] = 264735,
		["BindingShot"] = 109248,
		["DeathChakram"] = 375891,
		["ImprovedKillCommand"] = 378010,
		["LockandLoad"] = 194595,
		["LoneWolf"] = 155228,
		["Trailblazer"] = 199921,
		["BeastMaster"] = 378007,
		["NaturalMending"] = 270581,
		["Misdirection"] = 34477,
		["Posthaste"] = 109215,
		["Stampede"] = 201430,
		["EagletalonsTrueFocus"] = 389449,
		["SentinelOwl"] = 388045,
		["CounterShot"] = 147362,
		["ChimaeraShot"] = 342049,
		["SerratedShots"] = 389882,
		["RejuvenatingWind"] = 385539,
		["BurstingShot"] = 186387,
		["Deadeye"] = 321460,
		["FeignDeath"] = 5384,
		["Disengage"] = 781,
		["KillCommand"] = 34026,
		["Trueshot"] = 288613,
		["HighExplosiveTrap"] = 236776,
		["Bombardment"] = 405804,
		["SteelTrap"] = 162488,
		["EyesoftheBeast"] = 321297,
		["SteadyFocus"] = 193533,
		["HydrasBite"] = 260241,
		["Deathblow"] = 378769,
		["Sharpshooter"] = 378907,
		["RapidFire"] = 257044,
		["SurgingShots"] = 391559,
		["ExplosiveShot"] = 212431,
		["HeavyAmmo"] = 378910,
		["ArcticBola"] = 390231,
		},
		["Beast Mastery"] = {
		["PoisonInjection"] = 378014,
		["NaturesEndurance"] = 388042,
		["BrutalCompanion"] = 386870,
		["BeastCleave"] = 115939,
		["BindingShackles"] = 321468,
		["WildernessMedicine"] = 343242,
		["WailingArrow"] = 392060,
		["Fetch"] = 125050,
		["ImprovedKillShot"] = 343248,
		["ScareBeast"] = 1513,
		["MasterMarksman"] = 260309,
		["KindredSpirits"] = 56315,
		["SteelTrap"] = 162488,
		["SentinelsProtection"] = 388057,
		["DireFrenzy"] = 385810,
		["ConcussiveShot"] = 5116,
		["ScatterShot"] = 213691,
		["ThrilloftheHunt"] = 257944,
		["Posthaste"] = 109215,
		["Barrage"] = 120360,
		["MasterHandler"] = 424558,
		["SharpBarbs"] = 378205,
		["MultiShot"] = 2643,
		["Misdirection"] = 34477,
		["TranquilizingShot"] = 19801,
		["AMurderofCrows"] = 131894,
		["Pathfinding"] = 378002,
		["WildInstincts"] = 378442,
		["SentinelsPerception"] = 388056,
		["ImprovedTraps"] = 343247,
		["WingClip"] = 195645,
		["KillerInstinct"] = 273887,
		["Intimidation"] = 19577,
		["MasteryMasterofBeasts"] = 76657,
		["ExoticBeasts"] = 53270,
		["Exhilaration"] = 109304,
		["PlayDead"] = 209997,
		["KillCleave"] = 378207,
		["HuntersMark"] = 257284,
		["FreezingTrap"] = 187650,
		["Flare"] = 1543,
		["RoarofSacrifice"] = 53480,
		["FeignDeath"] = 5384,
		["KillCommand"] = 34026,
		["Savagery"] = 424557,
		["TarTrap"] = 187698,
		["KillerCobra"] = 199532,
		["EagleEye"] = 6197,
		["Entrapment"] = 393344,
		["Disengage"] = 781,
		["AspectoftheCheetah"] = 186257,
		["LoneSurvivor"] = 388039,
		["CommandPet"] = 272651,
		["KillShot"] = 53351,
		["AspectoftheTurtle"] = 186265,
		["SerpentSting"] = 271788,
		["KeenEyesight"] = 378004,
		["Trailblazer"] = 199921,
		["Stomp"] = 199530,
		["ArcaneShot"] = 185358,
		["CobraShot"] = 193455,
		["NaturalMending"] = 270581,
		["ImprovedTranquilizingShot"] = 343244,
		["BarbedShot"] = 217200,
		["BornToBeWild"] = 266921,
		["CalloftheWild"] = 359844,
		["WildCall"] = 185789,
		["ScentofBlood"] = 193532,
		["SurvivaloftheFittest"] = 264735,
		["BindingShot"] = 109248,
		["DeathChakram"] = 375891,
		["CobraSenses"] = 378244,
		["PiercingFangs"] = 392053,
		["ImprovedKillCommand"] = 378010,
		["OnewiththePack"] = 199528,
		["BloodyFrenzy"] = 407412,
		["PackTactics"] = 321014,
		["AnimalCompanion"] = 267116,
		["CobraSting"] = 378750,
		["WarOrders"] = 393933,
		["DireCommand"] = 378743,
		["SentinelOwl"] = 388045,
		["CounterShot"] = 147362,
		["Camouflage"] = 199483,
		["SerratedShots"] = 389882,
		["RejuvenatingWind"] = 385539,
		["SteadyShot"] = 56641,
		["BarbedWrath"] = 231548,
		["BestialWrath"] = 19574,
		["DireBeast"] = 120679,
		["DirePack"] = 378745,
		["KillerCommand"] = 378740,
		["HighExplosiveTrap"] = 236776,
		["AspectoftheBeast"] = 191384,
		["Bloodshed"] = 321530,
		["EyesoftheBeast"] = 321297,
		["Stampede"] = 201430,
		["HydrasBite"] = 260241,
		["HuntersPrey"] = 378210,
		["ArcticBola"] = 390231,
		["AlphaPredator"] = 269737,
		["HuntersAvoidance"] = 384799,
		["ExplosiveShot"] = 212431,
		["BeastMaster"] = 378007,
		["TrainingExpert"] = 378209,
		},
		["Survival"] = {
		["PrimalRage"] = 272678,
		["PoisonInjection"] = 378014,
		["NaturesEndurance"] = 388042,
		["BindingShackles"] = 321468,
		["WildernessMedicine"] = 343242,
		["ImprovedWildfireBomb"] = 321290,
		["Fetch"] = 125050,
		["ImprovedKillShot"] = 343248,
		["ScareBeast"] = 1513,
		["MasterMarksman"] = 260309,
		["SteelTrap"] = 162488,
		["SentinelsProtection"] = 388057,
		["SweepingSpear"] = 378950,
		["ConcussiveShot"] = 5116,
		["ScatterShot"] = 213691,
		["ArcaneShot"] = 185358,
		["Posthaste"] = 109215,
		["RuthlessMarauder"] = 385718,
		["Bloodseeker"] = 260248,
		["WildfireInfusion"] = 271014,
		["TacticalAdvantage"] = 378951,
		["CoordinatedAssault"] = 360952,
		["Stampede"] = 201430,
		["BirdsofPrey"] = 260331,
		["KillerCompanion"] = 378955,
		["TranquilizingShot"] = 19801,
		["Pathfinding"] = 378002,
		["SentinelsPerception"] = 388056,
		["MongooseBite"] = 259387,
		["Camouflage"] = 199483,
		["MasterySpiritBond"] = 263135,
		["KillerInstinct"] = 273887,
		["ShrapnelBomb"] = 270335,
		["Intimidation"] = 19577,
		["WingClip"] = 195645,
		["SteadyShot"] = 56641,
		["Exhilaration"] = 109304,
		["CommandPet"] = 272651,
		["PlayDead"] = 209997,
		["HuntersMark"] = 257284,
		["SpearFocus"] = 378953,
		["FlankingStrike"] = 269751,
		["RoarofSacrifice"] = 53480,
		["Flare"] = 1543,
		["KillCommand"] = 259489,
		["FreezingTrap"] = 187650,
		["QuickShot"] = 378940,
		["VipersVenom"] = 268501,
		["EagleEye"] = 6197,
		["Entrapment"] = 393344,
		["Muzzle"] = 187707,
		["AspectoftheCheetah"] = 186257,
		["LoneSurvivor"] = 388039,
		["Disengage"] = 781,
		["Butchery"] = 212436,
		["Spearhead"] = 360966,
		["SerpentSting"] = 271788,
		["KeenEyesight"] = 378004,
		["Trailblazer"] = 199921,
		["AspectoftheTurtle"] = 186265,
		["TarTrap"] = 187698,
		["SharpEdges"] = 378948,
		["NaturalMending"] = 270581,
		["ImprovedTranquilizingShot"] = 343244,
		["BornToBeWild"] = 266921,
		["TermsofEngagement"] = 265895,
		["Carve"] = 187708,
		["Misdirection"] = 34477,
		["AspectoftheEagle"] = 186289,
		["SurvivaloftheFittest"] = 264735,
		["BindingShot"] = 109248,
		["DeathChakram"] = 375891,
		["Bombardier"] = 389880,
		["IntenseFocus"] = 385709,
		["ImprovedKillCommand"] = 378010,
		["Lunge"] = 378934,
		["ImprovedTraps"] = 343247,
		["Barrage"] = 120360,
		["Ferocity"] = 378916,
		["Harpoon"] = 190925,
		["FuryoftheEagle"] = 203415,
		["HighExplosiveTrap"] = 236776,
		["SentinelOwl"] = 388045,
		["KillShot"] = 320976,
		["Ranger"] = 385695,
		["SerratedShots"] = 389882,
		["RejuvenatingWind"] = 385539,
		["ExplosivesExpert"] = 378937,
		["TipoftheSpear"] = 260285,
		["FeignDeath"] = 5384,
		["BloodyClaws"] = 385737,
		["FrenzyStrikes"] = 294029,
		["EnergeticAlly"] = 378961,
		["RaptorStrike"] = 186270,
		["WildfireBomb"] = 259495,
		["DeadlyDuo"] = 378962,
		["EyesoftheBeast"] = 321297,
		["GuerrillaTactics"] = 264332,
		["HydrasBite"] = 260241,
		["HuntersAvoidance"] = 384799,
		["ArcticBola"] = 390231,
		["AlphaPredator"] = 269737,
		["FlankersAdvantage"] = 263186,
		["ExplosiveShot"] = 212431,
		["BeastMaster"] = 378007,
		["CoordinatedKill"] = 385739,
		},
		},
		["Evoker"] = {
		["Devastation"] = {
		["FocusingIris"] = 386336,
		["TwinGuardian"] = 370888,
		["ArcaneIntensity"] = 375618,
		["ClobberingSweep"] = 375443,
		["Zephyr"] = 374227,
		["Tailwind"] = 375556,
		["EssenceAttunement"] = 375722,
		["ProtractedTalons"] = 369909,
		["ChargedBlast"] = 370455,
		["PowerSwell"] = 370839,
		["CauterizingFlame"] = 374251,
		["BlastFurnace"] = 375510,
		["Snapfire"] = 370783,
		["HoardedPower"] = 375796,
		["LayWaste"] = 371034,
		["ObsidianBulwark"] = 375406,
		["BlessingoftheBronze"] = 364342,
		["AncientFlame"] = 369990,
		["Iridescence"] = 370867,
		["InnerRadiance"] = 386405,
		["WallopingBlow"] = 387341,
		["Burnout"] = 375801,
		["EyeofInfinity"] = 411165,
		["DraconicLegacy"] = 376166,
		["FociofLife"] = 375574,
		["PermeatingChill"] = 370897,
		["PowerNexus"] = 369908,
		["Recall"] = 371806,
		["DeepBreath"] = 357210,
		["Firestorm"] = 368847,
		["TitanicWrath"] = 386272,
		["EssenceBurst"] = 359565,
		["SleepWalk"] = 360806,
		["RagingInferno"] = 405659,
		["NaturalConvergence"] = 369913,
		["HeatWave"] = 375725,
		["Unravel"] = 368432,
		["EventHorizon"] = 411164,
		["ImminentDestruction"] = 370781,
		["LivingFlame"] = 361469,
		["Pyre"] = 357211,
		["FuryoftheAspects"] = 390386,
		["RenewingBlaze"] = 374348,
		["ObsidianScales"] = 363916,
		["Overawe"] = 374346,
		["ArcaneVigor"] = 386342,
		["AttunedtotheDream"] = 376930,
		["DenseEnergy"] = 370962,
		["RubyEmbers"] = 365937,
		["SpellweaversDominance"] = 370845,
		["EternitysSpan"] = 375757,
		["InstinctiveArcana"] = 376164,
		["HonedAggression"] = 371038,
		["PotentMana"] = 418101,
		["Rescue"] = 370665,
		["Causality"] = 375777,
		["Animosity"] = 375797,
		["Hover"] = 358267,
		["VerdantEmbrace"] = 360995,
		["Return"] = 361227,
		["ShatteringStar"] = 370452,
		["OnyxLegacy"] = 386348,
		["RubyEssenceBurst"] = 376872,
		["TimeSpiral"] = 374968,
		["Tyranny"] = 376888,
		["Expunge"] = 365585,
		["InnateMagic"] = 375520,
		["Panacea"] = 387761,
		["FireWithin"] = 375577,
		["Enkindled"] = 375554,
		["TiptheScales"] = 370553,
		["LeapingFlames"] = 369939,
		["LushGrowth"] = 375561,
		["BountifulBloom"] = 370886,
		["ForgerofMountains"] = 375528,
		["ExtendedFlight"] = 375517,
		["Dragonrage"] = 375087,
		["AerialMastery"] = 365933,
		["Volatility"] = 369089,
		["OppressingRoar"] = 372048,
		["ImposingPresence"] = 371016,
		["AzureEssenceBurst"] = 375721,
		["Catalyze"] = 386283,
		["EmeraldBlossom"] = 355913,
		["Scintillation"] = 370821,
		["FontofMagic"] = 411212,
		["TerroroftheSkies"] = 371032,
		["ScarletAdaptation"] = 372469,
		["RegenerativeMagic"] = 387787,
		["SourceofMagic"] = 369459,
		["EngulfingBlaze"] = 370837,
		["Exuberance"] = 375542,
		["EnergizingFlame"] = 400006,
		["EternitySurge"] = 359073,
		["Quell"] = 351338,
		["EverburningFlame"] = 370819,
		["TemperedScales"] = 396571,
		["Landslide"] = 358385,
		["InherentResistance"] = 375544,
		["FireBreath"] = 357208,
		["HeavyWingbeats"] = 368838,
		["FeedtheFlames"] = 369846,
		["AzureStrike"] = 362969,
		["Disintegrate"] = 356995,
		["MasteryGiantkiller"] = 362980,
		},
		["Augmentation"] = {
		["SourceofMagic"] = 369459,
		["StretchTime"] = 410352,
		["MotesofPossibility"] = 409267,
		["TwinGuardian"] = 370888,
		["AspectsFavor"] = 407243,
		["ClobberingSweep"] = 375443,
		["Zephyr"] = 374227,
		["Tailwind"] = 375556,
		["InfernosBlessing"] = 410261,
		["EssenceAttunement"] = 375722,
		["EchoingStrike"] = 410784,
		["CauterizingFlame"] = 374251,
		["Prescience"] = 409311,
		["BlastFurnace"] = 375510,
		["PlottheFuture"] = 407866,
		["HoardedPower"] = 375796,
		["ObsidianBulwark"] = 375406,
		["BlessingoftheBronze"] = 364342,
		["AncientFlame"] = 369990,
		["BronzeAttunement"] = 403265,
		["LivingFlame"] = 361469,
		["BlisteringScales"] = 360827,
		["FireBreath"] = 357208,
		["Timelessness"] = 412710,
		["ImposingPresence"] = 371016,
		["MasteryTimewalker"] = 406380,
		["Anachronism"] = 407869,
		["FociofLife"] = 375574,
		["PermeatingChill"] = 370897,
		["PowerNexus"] = 369908,
		["AttunedtotheDream"] = 376930,
		["MomentumShift"] = 408004,
		["Recall"] = 371806,
		["DeepBreath"] = 357210,
		["Rescue"] = 370665,
		["SpatialParadox"] = 406732,
		["MoltenBlood"] = 410643,
		["NourishingSands"] = 406041,
		["InterwovenThreads"] = 412713,
		["EssenceBurst"] = 396187,
		["TemperedScales"] = 396571,
		["SleepWalk"] = 360806,
		["Quell"] = 351338,
		["NaturalConvergence"] = 369913,
		["Landslide"] = 358385,
		["Unravel"] = 368432,
		["PerilousFate"] = 410253,
		["WallopingBlow"] = 387341,
		["Return"] = 361227,
		["FireWithin"] = 375577,
		["FuryoftheAspects"] = 390386,
		["RenewingBlaze"] = 374348,
		["ObsidianScales"] = 363916,
		["Overawe"] = 374346,
		["DraconicAttunements"] = 403208,
		["DraconicLegacy"] = 376166,
		["Panacea"] = 387761,
		["Overlord"] = 410260,
		["BreathofEons"] = 403631,
		["RicochetingPyroclast"] = 406659,
		["Exuberance"] = 375542,
		["BestowWeyrnstone"] = 408233,
		["SandsofTime"] = 395153,
		["CloseasClutchmates"] = 396043,
		["SeismicSlam"] = 408543,
		["Eruption"] = 395160,
		["TerroroftheSkies"] = 371032,
		["VerdantEmbrace"] = 360995,
		["FontofMagic"] = 408083,
		["LeapingFlames"] = 369939,
		["Geomancy"] = 410787,
		["EbonMight"] = 395152,
		["TimeSpiral"] = 374968,
		["DefyFate"] = 404195,
		["Enkindled"] = 375554,
		["InnateMagic"] = 375520,
		["BlackAttunement"] = 403264,
		["Accretion"] = 407876,
		["Upheaval"] = 396286,
		["TiptheScales"] = 370553,
		["Volcanism"] = 406904,
		["LushGrowth"] = 375561,
		["ChronoWard"] = 409676,
		["SymbioticBloom"] = 410685,
		["ExtendedFlight"] = 375517,
		["Expunge"] = 365585,
		["AerialMastery"] = 365933,
		["InstinctiveArcana"] = 376164,
		["OppressingRoar"] = 372048,
		["BountifulBloom"] = 370886,
		["IgnitionRush"] = 408775,
		["ForgerofMountains"] = 375528,
		["EmeraldBlossom"] = 355913,
		["PupilofAlexstrasza"] = 407814,
		["DreamofSpring"] = 414969,
		["Hover"] = 358267,
		["ScarletAdaptation"] = 372469,
		["ReactiveHide"] = 409329,
		["RegenerativeMagic"] = 387787,
		["TimeSkip"] = 404977,
		["FateMirror"] = 412774,
		["EnergizingFlame"] = 400006,
		["ProtractedTalons"] = 369909,
		["ProlongLife"] = 410687,
		["PotentMana"] = 418101,
		["RegenerativeChitin"] = 406907,
		["UnyieldingDomain"] = 412733,
		["InherentResistance"] = 375544,
		["TomorrowToday"] = 412723,
		["HeavyWingbeats"] = 368838,
		["TectonicLocus"] = 408002,
		["AzureStrike"] = 362969,
		["Disintegrate"] = 356995,
		["InnerRadiance"] = 386405,
		},
		["Preservation"] = {
		["Stasis"] = 370537,
		["Erasure"] = 376210,
		["TwinGuardian"] = 370888,
		["Rescue"] = 370665,
		["GoldenHour"] = 378196,
		["Tailwind"] = 375556,
		["EssenceAttunement"] = 375722,
		["AttunedtotheDream"] = 376930,
		["EmeraldCommunion"] = 370960,
		["CauterizingFlame"] = 374251,
		["Ouroboros"] = 381921,
		["ExhilaratingBurst"] = 377100,
		["TimelessMagic"] = 376240,
		["ObsidianBulwark"] = 375406,
		["BlessingoftheBronze"] = 364342,
		["Lifebind"] = 373270,
		["AncientFlame"] = 369990,
		["LivingFlame"] = 361469,
		["WallopingBlow"] = 387341,
		["TimeLord"] = 372527,
		["Dreamwalker"] = 377082,
		["DraconicLegacy"] = 376166,
		["SpiritualClarity"] = 376150,
		["PermeatingChill"] = 370897,
		["PowerNexus"] = 369908,
		["Recall"] = 371806,
		["DeepBreath"] = 357210,
		["FireBreath"] = 357208,
		["TemporalAnomaly"] = 373861,
		["EnergyLoop"] = 372233,
		["SourceofMagic"] = 369459,
		["LifeGiversFlame"] = 371426,
		["EssenceBurst"] = 369297,
		["PotentMana"] = 418101,
		["SleepWalk"] = 360806,
		["LeapingFlames"] = 369939,
		["NaturalConvergence"] = 369913,
		["Landslide"] = 358385,
		["Unravel"] = 368432,
		["TemperedScales"] = 396571,
		["DreamBreath"] = 355936,
		["MassReturn"] = 361178,
		["FireWithin"] = 375577,
		["FuryoftheAspects"] = 390386,
		["RenewingBlaze"] = 374348,
		["ObsidianScales"] = 363916,
		["Overawe"] = 374346,
		["NozdormusTeachings"] = 376237,
		["CycleofLife"] = 371832,
		["CallofYsera"] = 373834,
		["TerroroftheSkies"] = 371032,
		["Echo"] = 364343,
		["RenewingBreath"] = 371257,
		["Exuberance"] = 375542,
		["LifeforceMender"] = 376179,
		["Return"] = 361227,
		["RushofVitality"] = 377086,
		["ResonatingSphere"] = 376236,
		["TimeDilation"] = 357170,
		["Hover"] = 358267,
		["VerdantEmbrace"] = 360995,
		["RegenerativeMagic"] = 387787,
		["TimeofNeed"] = 368412,
		["ProtractedTalons"] = 369909,
		["BlastFurnace"] = 375510,
		["TimeSpiral"] = 374968,
		["Expunge"] = 365585,
		["InstinctiveArcana"] = 376164,
		["InnateMagic"] = 375520,
		["Reversion"] = 366155,
		["FociofLife"] = 375574,
		["TiptheScales"] = 370553,
		["Enkindled"] = 375554,
		["DreamFlight"] = 359816,
		["LushGrowth"] = 375561,
		["BountifulBloom"] = 370886,
		["SparkofInsight"] = 377099,
		["ExtendedFlight"] = 375517,
		["Punctuality"] = 371270,
		["AerialMastery"] = 365933,
		["JustinTime"] = 376204,
		["OppressingRoar"] = 372048,
		["FlowState"] = 385696,
		["Rewind"] = 363534,
		["ForgerofMountains"] = 375528,
		["EmeraldBlossom"] = 355913,
		["GracePeriod"] = 376239,
		["FontofMagic"] = 375783,
		["Naturalize"] = 360823,
		["ScarletAdaptation"] = 372469,
		["ClobberingSweep"] = 375443,
		["Zephyr"] = 374227,
		["Panacea"] = 387761,
		["MasteryLifeBinder"] = 363510,
		["TemporalCompression"] = 362874,
		["Spiritbloom"] = 367226,
		["Quell"] = 351338,
		["TemporalArtificer"] = 381922,
		["FieldofDreams"] = 370062,
		["EnergizingFlame"] = 400006,
		["InherentResistance"] = 375544,
		["Empath"] = 376138,
		["HeavyWingbeats"] = 368838,
		["FlutteringSeedlings"] = 359793,
		["AzureStrike"] = 362969,
		["Disintegrate"] = 356995,
		["DelayHarm"] = 376207,
		},
		},
		["Druid"] = {
		["Feral"] = {
		["Swiftmend"] = 18562,
		["MightyBash"] = 5211,
		["TravelForm"] = 783,
		["FranticMomentum"] = 391875,
		["Innervate"] = 29166,
		["Dash"] = 1850,
		["SouloftheForest"] = 158476,
		["CatForm"] = 768,
		["Revive"] = 50769,
		["ImprovedBarkskin"] = 327993,
		["UrsineVigor"] = 377842,
		["AdaptiveSwarm"] = 391888,
		["Sabertooth"] = 202031,
		["WildGrowth"] = 48438,
		["Hibernate"] = 2637,
		["FelineAdept"] = 300349,
		["RipandTear"] = 391347,
		["Starfire"] = 197628,
		["ImprovedSunfire"] = 231050,
		["MassEntanglement"] = 102359,
		["FerociousBite"] = 22568,
		["Predator"] = 202021,
		["CircleofLifeandDeath"] = 400320,
		["DireFixation"] = 417710,
		["Moonfire"] = 8921,
		["UrsolsVortex"] = 102793,
		["WellHonedInstincts"] = 377847,
		["Thrash"] = 106832,
		["PouncingStrikes"] = 390772,
		["ImprovedSwipe"] = 400158,
		["Cyclone"] = 33786,
		["SkullBash"] = 106839,
		["Barkskin"] = 22812,
		["ConvoketheSpirits"] = 391528,
		["Regrowth"] = 8936,
		["MercilessClaws"] = 231063,
		["TreantForm"] = 114282,
		["FlightForm"] = 276029,
		["Maim"] = 22570,
		["WildCharge"] = 102401,
		["RagingFury"] = 391078,
		["Sunfire"] = 93402,
		["LunarInspiration"] = 155580,
		["BerserkFrenzy"] = 384668,
		["CharmWoodlandCreature"] = 127757,
		["GaleWinds"] = 400142,
		["RemoveCorruption"] = 2782,
		["SuddenAmbush"] = 384667,
		["AquaticForm"] = 276012,
		["Rake"] = 1822,
		["Rip"] = 1079,
		["DoubleClawedRake"] = 391700,
		["KillerInstinct"] = 108299,
		["Berserk"] = 106951,
		["Starsurge"] = 197626,
		["MountForm"] = 210053,
		["VerdantHeart"] = 301768,
		["Wrath"] = 5176,
		["Dreamwalk"] = 193753,
		["EntanglingRoots"] = 339,
		["Rejuvenation"] = 774,
		["FeralInstinct"] = 16949,
		["TirelessEnergy"] = 383352,
		["IncapacitatingRoar"] = 99,
		["CarnivorousInstinct"] = 390902,
		["ProtectorofthePack"] = 378986,
		["FrenziedRegeneration"] = 22842,
		["PrimalWrath"] = 285381,
		["Shred"] = 5221,
		["LionsStrength"] = 391972,
		["MattedFur"] = 385786,
		["RelentlessPredator"] = 393771,
		["RampantFerocity"] = 391709,
		["HeartoftheWild"] = 319454,
		["FelineSwiftness"] = 131768,
		["StampedingRoar"] = 106898,
		["BearForm"] = 5487,
		["UnbridledSwarm"] = 391951,
		["MasteryRazorClaws"] = 77493,
		["AshamanesGuidance"] = 391548,
		["NaturalRecovery"] = 377796,
		["IncarnationAvatarofAshamane"] = 102543,
		["DreadfulBleeding"] = 391045,
		["MoonkinForm"] = 197625,
		["TigerDash"] = 252216,
		["Rebirth"] = 20484,
		["PrimalFury"] = 159286,
		["Veinripper"] = 391978,
		["TigersFury"] = 5217,
		["Ironfur"] = 192081,
		["RisingLightFallingNight"] = 417712,
		["Prowl"] = 5215,
		["TearOpenWounds"] = 391785,
		["PredatorySwiftness"] = 16974,
		["InfectedWounds"] = 48484,
		["TirelessPursuit"] = 377801,
		["TasteforBlood"] = 384665,
		["SurvivalInstincts"] = 61336,
		["MomentofClarity"] = 236068,
		["LycarasTeachings"] = 378988,
		["ApexPredatorsCraving"] = 391881,
		["ThickHide"] = 16931,
		["Renewal"] = 108238,
		["Growl"] = 6795,
		["ImprovedRejuvenation"] = 231040,
		["ImprovedStampedingRoar"] = 288826,
		["Typhoon"] = 132469,
		["SaberJaws"] = 421432,
		["Forestwalk"] = 400129,
		["Mangle"] = 33917,
		["Swipe"] = 213764,
		["Bloodtalons"] = 319439,
		["WildSlashes"] = 390864,
		["TigersTenacity"] = 391872,
		["NurturingInstinct"] = 33873,
		["BrutalSlash"] = 202028,
		["Soothe"] = 2908,
		["Flap"] = 164862,
		["FeralFrenzy"] = 274837,
		["ThrashingClaws"] = 405300,
		["OmenofClarity"] = 16864,
		["AstralInfluence"] = 197524,
		["NaturesVigil"] = 124974,
		["IncessantTempest"] = 400140,
		["BerserkHeartoftheLion"] = 391174,
		["ProtectiveGrowth"] = 391947,
		["MarkoftheWild"] = 1126,
		},
		["Balance"] = {
		["AstralSmolder"] = 394058,
		["NurturingInstinct"] = 33873,
		["MightyBash"] = 5211,
		["NaturesGrace"] = 393958,
		["SouloftheForest"] = 114107,
		["ImprovedBarkskin"] = 327993,
		["UrsineVigor"] = 377842,
		["Mangle"] = 33917,
		["SolarBeam"] = 78675,
		["WildGrowth"] = 48438,
		["Hibernate"] = 2637,
		["AstralCommunion"] = 202359,
		["Starfire"] = 194153,
		["ImprovedSunfire"] = 231050,
		["MassEntanglement"] = 102359,
		["Rake"] = 1822,
		["StellarFlare"] = 202347,
		["HeartoftheWild"] = 319454,
		["LycarasTeachings"] = 378988,
		["IncessantTempest"] = 400140,
		["UrsolsVortex"] = 102793,
		["Revive"] = 50769,
		["Thrash"] = 106832,
		["OwlkinFrenzy"] = 231042,
		["PrimalFury"] = 159286,
		["Cyclone"] = 33786,
		["SkullBash"] = 106839,
		["Maim"] = 22570,
		["Typhoon"] = 132469,
		["NaturesVigil"] = 124974,
		["AstralInfluence"] = 197524,
		["OrbitBreaker"] = 383197,
		["FlightForm"] = 276029,
		["ShootingStars"] = 202342,
		["WildCharge"] = 102401,
		["Moonfire"] = 8921,
		["IncarnationChosenofElune"] = 390414,
		["Solstice"] = 343647,
		["ImprovedRejuvenation"] = 231040,
		["CharmWoodlandCreature"] = 127757,
		["GaleWinds"] = 400142,
		["RemoveCorruption"] = 2782,
		["BalanceofAllThings"] = 394048,
		["AquaticForm"] = 276012,
		["UmbralIntensity"] = 383195,
		["Rip"] = 1079,
		["Starfall"] = 191034,
		["MarkoftheWild"] = 1126,
		["Dreamwalk"] = 193753,
		["Starsurge"] = 78674,
		["MountForm"] = 210053,
		["ConvoketheSpirits"] = 391528,
		["Wrath"] = 5176,
		["Sunfire"] = 93402,
		["Barkskin"] = 22812,
		["Rejuvenation"] = 774,
		["Regrowth"] = 8936,
		["Forestwalk"] = 400129,
		["Starlord"] = 202345,
		["WildSurges"] = 406890,
		["ProtectorofthePack"] = 378986,
		["FrenziedRegeneration"] = 22842,
		["LightoftheSun"] = 202918,
		["RattletheStars"] = 393954,
		["Starweaver"] = 393940,
		["MattedFur"] = 385786,
		["WaningTwilight"] = 393956,
		["NaturesBalance"] = 202430,
		["VerdantHeart"] = 301768,
		["CosmicRapidity"] = 400059,
		["StampedingRoar"] = 106898,
		["MasteryAstralInvocation"] = 393014,
		["FriendoftheFae"] = 394081,
		["DenizenoftheDream"] = 394065,
		["ForceofNature"] = 205636,
		["NaturalRecovery"] = 377796,
		["Shred"] = 5221,
		["EntanglingRoots"] = 339,
		["WildMushroom"] = 88747,
		["TigerDash"] = 252216,
		["Rebirth"] = 20484,
		["BearForm"] = 5487,
		["Ironfur"] = 192081,
		["AetherialKindling"] = 327541,
		["MoonkinForm"] = 24858,
		["Flap"] = 164862,
		["StellarInnervation"] = 394115,
		["NewMoon"] = 274281,
		["CelestialAlignment"] = 194223,
		["UmbralEmbrace"] = 393760,
		["OrbitalStrike"] = 390378,
		["Prowl"] = 5215,
		["TirelessPursuit"] = 377801,
		["RisingLightFallingNight"] = 417712,
		["PrimordialArcanicPulsar"] = 393960,
		["Eclipse"] = 79577,
		["ThickHide"] = 16931,
		["WarriorofElune"] = 202425,
		["Growl"] = 6795,
		["LunarShrapnel"] = 415169,
		["PowerofGoldrinn"] = 394046,
		["ImprovedSwipe"] = 400158,
		["IncapacitatingRoar"] = 99,
		["Swiftmend"] = 18562,
		["TwinMoons"] = 279620,
		["Swipe"] = 213764,
		["FuryofElune"] = 202770,
		["ElunesGuidance"] = 393991,
		["TreantForm"] = 114282,
		["CatForm"] = 768,
		["ImprovedStampedingRoar"] = 288826,
		["Soothe"] = 2908,
		["RadiantMoonlight"] = 394121,
		["SunderedFirmament"] = 394094,
		["Renewal"] = 108238,
		["FelineSwiftness"] = 131768,
		["KillerInstinct"] = 108299,
		["WellHonedInstincts"] = 377847,
		["Dash"] = 1850,
		["Innervate"] = 29166,
		["FerociousBite"] = 22568,
		["TravelForm"] = 783,
		},
		["Guardian"] = {
		["Swiftmend"] = 18562,
		["ElunesFavored"] = 370586,
		["TravelForm"] = 783,
		["ScintillatingMoonlight"] = 238049,
		["UrsocsGuidance"] = 393414,
		["Innervate"] = 29166,
		["ImprovedBarkskin"] = 327993,
		["ReinforcedFur"] = 393618,
		["Mangle"] = 33917,
		["IncessantTempest"] = 400140,
		["WildGrowth"] = 48438,
		["Hibernate"] = 2637,
		["FerociousBite"] = 22568,
		["FelineSwiftness"] = 131768,
		["Starfire"] = 197628,
		["MoonlessNight"] = 400278,
		["MassEntanglement"] = 102359,
		["NaturesVigil"] = 124974,
		["LycarasTeachings"] = 378988,
		["CircleofLifeandDeath"] = 391969,
		["VulnerableFlesh"] = 372618,
		["SkullBash"] = 106839,
		["UrsolsVortex"] = 102793,
		["Moonfire"] = 8921,
		["RendandTear"] = 204053,
		["Reinvigoration"] = 372945,
		["ImprovedSwipe"] = 400158,
		["Cyclone"] = 33786,
		["Brambles"] = 203953,
		["UrsineAdept"] = 300346,
		["WellHonedInstincts"] = 377847,
		["Regrowth"] = 8936,
		["AstralInfluence"] = 197524,
		["Dreamwalk"] = 193753,
		["FlightForm"] = 276029,
		["Earthwarden"] = 203974,
		["WildCharge"] = 102401,
		["Maim"] = 22570,
		["CatForm"] = 768,
		["MarkoftheWild"] = 1126,
		["Swipe"] = 213764,
		["CharmWoodlandCreature"] = 127757,
		["BloodFrenzy"] = 203962,
		["RemoveCorruption"] = 2782,
		["Barkskin"] = 22812,
		["AquaticForm"] = 276012,
		["Rake"] = 1822,
		["Rip"] = 1079,
		["Pulverize"] = 80313,
		["KillerInstinct"] = 108299,
		["Berserk"] = 50334,
		["Starsurge"] = 197626,
		["MountForm"] = 210053,
		["ConvoketheSpirits"] = 391528,
		["Wrath"] = 5176,
		["ImprovedSunfire"] = 231050,
		["Sunfire"] = 93402,
		["Rejuvenation"] = 774,
		["IncapacitatingRoar"] = 99,
		["Forestwalk"] = 400129,
		["MasteryNaturesGuardian"] = 155783,
		["LunarBeam"] = 204066,
		["ProtectorofthePack"] = 378986,
		["FrenziedRegeneration"] = 22842,
		["LightningReflexes"] = 231065,
		["UrsocsEndurance"] = 393611,
		["Growl"] = 6795,
		["MattedFur"] = 385786,
		["HeartoftheWild"] = 319454,
		["BearForm"] = 5487,
		["GalacticGuardian"] = 203964,
		["DreamofCenarius"] = 372119,
		["StampedingRoar"] = 106898,
		["SouloftheForest"] = 158477,
		["Flap"] = 164862,
		["GuardianofElune"] = 155578,
		["RisingLightFallingNight"] = 417712,
		["NaturalRecovery"] = 377796,
		["Shred"] = 5221,
		["EntanglingRoots"] = 339,
		["TirelessPursuit"] = 377801,
		["TigerDash"] = 252216,
		["Rebirth"] = 20484,
		["UrsineVigor"] = 377842,
		["IncarnationGuardianofUrsoc"] = 102558,
		["Ironfur"] = 192081,
		["BristlingFur"] = 155835,
		["SurvivaloftheFittest"] = 203965,
		["GoryFur"] = 200854,
		["AftertheWildfire"] = 371905,
		["MoonkinForm"] = 197625,
		["InfectedWounds"] = 345208,
		["SurvivalInstincts"] = 61336,
		["Prowl"] = 5215,
		["MightyBash"] = 5211,
		["PrimalFury"] = 159286,
		["ImprovedRejuvenation"] = 231040,
		["Renewal"] = 108238,
		["ThickHide"] = 16931,
		["ViciousCycle"] = 371999,
		["ThornsofIron"] = 400222,
		["ImprovedStampedingRoar"] = 288826,
		["GaleWinds"] = 400142,
		["FlashingClaws"] = 393427,
		["Maul"] = 6807,
		["RageoftheSleeper"] = 200851,
		["NurturingInstinct"] = 33873,
		["InnateResolve"] = 377811,
		["Raze"] = 400254,
		["VerdantHeart"] = 301768,
		["TreantForm"] = 114282,
		["FuryofNature"] = 370695,
		["ToothandClaw"] = 135288,
		["Soothe"] = 2908,
		["UntamedSavagery"] = 372943,
		["Revive"] = 50769,
		["Typhoon"] = 132469,
		["Thrash"] = 106832,
		["UrsocsFury"] = 377210,
		["LayeredMane"] = 384721,
		["Gore"] = 210706,
		["TwinMoonfire"] = 372567,
		["Dash"] = 1850,
		["ImprovedSurvivalInstincts"] = 328767,
		},
		["Restoration"] = {
		["Swiftmend"] = 18562,
		["MightyBash"] = 5211,
		["WildSynthesis"] = 400533,
		["SouloftheForest"] = 158478,
		["CenariusGuidance"] = 393371,
		["ImprovedBarkskin"] = 327993,
		["UrsineVigor"] = 377842,
		["AdaptiveSwarm"] = 391888,
		["Reforestation"] = 392356,
		["Hibernate"] = 2637,
		["HeartoftheWild"] = 319454,
		["Thrash"] = 106832,
		["ImprovedSwipe"] = 400158,
		["Cyclone"] = 33786,
		["SkullBash"] = 106839,
		["PassingSeasons"] = 382550,
		["NaturesVigil"] = 124974,
		["AstralInfluence"] = 197524,
		["FlightForm"] = 276029,
		["WildCharge"] = 102401,
		["CharmWoodlandCreature"] = 127757,
		["RemoveCorruption"] = 440015,
		["AquaticForm"] = 276012,
		["HarmoniousBlooming"] = 392256,
		["Maim"] = 22570,
		["Starsurge"] = 197626,
		["MountForm"] = 210053,
		["BuddingLeaves"] = 392167,
		["Sunfire"] = 93402,
		["Regenesis"] = 383191,
		["IncapacitatingRoar"] = 99,
		["Photosynthesis"] = 274902,
		["RegenerativeHeartwood"] = 392116,
		["MasteryHarmony"] = 77495,
		["MattedFur"] = 385786,
		["StampedingRoar"] = 106898,
		["Efflorescence"] = 145205,
		["UnbridledSwarm"] = 391951,
		["TranquilMind"] = 403521,
		["TigerDash"] = 252216,
		["Abundance"] = 207383,
		["Nourish"] = 50464,
		["ThickHide"] = 16931,
		["UnstoppableGrowth"] = 382559,
		["Swipe"] = 213764,
		["Verdancy"] = 392325,
		["Liveliness"] = 426702,
		["Cultivation"] = 200390,
		["Moonfire"] = 8921,
		["FerociousBite"] = 22568,
		["NaturesSplendor"] = 392288,
		["Ironbark"] = 102342,
		["VerdantInfusion"] = 392410,
		["TravelForm"] = 783,
		["Mangle"] = 33917,
		["WildGrowth"] = 48438,
		["Stonebark"] = 197061,
		["ImprovedNaturesCure"] = 392378,
		["RampantGrowth"] = 404521,
		["Starfire"] = 197628,
		["ImprovedSunfire"] = 231050,
		["MassEntanglement"] = 102359,
		["ImprovedRegrowth"] = 231032,
		["FlashofClarity"] = 392220,
		["UrsolsVortex"] = 102793,
		["GroveTending"] = 383192,
		["PrimalFury"] = 159286,
		["CenarionWard"] = 102351,
		["Regrowth"] = 8936,
		["NurturingDormancy"] = 392099,
		["Overgrowth"] = 203651,
		["Invigorate"] = 392160,
		["YserasGift"] = 145108,
		["GaleWinds"] = 400142,
		["Rake"] = 1822,
		["Rip"] = 1079,
		["EmbraceoftheDream"] = 392124,
		["KillerInstinct"] = 108299,
		["Barkskin"] = 22812,
		["ImprovedStampedingRoar"] = 288826,
		["Wrath"] = 5176,
		["Rejuvenation"] = 774,
		["ProtectorofthePack"] = 378986,
		["FrenziedRegeneration"] = 22842,
		["Undergrowth"] = 392301,
		["Revitalize"] = 212040,
		["NaturesCure"] = 88423,
		["NurturingInstinct"] = 33873,
		["NaturalRecovery"] = 377796,
		["Shred"] = 5221,
		["EntanglingRoots"] = 339,
		["LycarasTeachings"] = 378988,
		["Innervate"] = 29166,
		["Rebirth"] = 20484,
		["SpringBlossoms"] = 207385,
		["MarkoftheWild"] = 1126,
		["Germination"] = 155675,
		["Ironfur"] = 192081,
		["Renewal"] = 108238,
		["IncessantTempest"] = 400140,
		["MasterShapeshifter"] = 289237,
		["WakingDream"] = 392221,
		["Flourish"] = 197721,
		["RisingLightFallingNight"] = 417712,
		["Prowl"] = 5215,
		["Soothe"] = 2908,
		["Tranquility"] = 740,
		["PoweroftheArchdruid"] = 392302,
		["TirelessPursuit"] = 377801,
		["BearForm"] = 5487,
		["Growl"] = 6795,
		["NaturesSwiftness"] = 132158,
		["FelineSwiftness"] = 131768,
		["GroveGuardians"] = 102693,
		["ImprovedWildGrowth"] = 328025,
		["Dreamstate"] = 392162,
		["CatForm"] = 768,
		["ConvoketheSpirits"] = 391528,
		["OmenofClarity"] = 113043,
		["Forestwalk"] = 400129,
		["LuxuriantSoil"] = 392315,
		["TreantForm"] = 114282,
		["Lifebloom"] = 33763,
		["Dreamwalk"] = 193753,
		["ImprovedRejuvenation"] = 231040,
		["Flap"] = 164862,
		["InnerPeace"] = 197073,
		["Typhoon"] = 132469,
		["MoonkinForm"] = 197625,
		["ImprovedIronbark"] = 382552,
		["WellHonedInstincts"] = 377847,
		["Dash"] = 1850,
		["Revive"] = 50769,
		["VerdantHeart"] = 301768,
		["IncarnationTreeofLife"] = 33891,
		},
		},
		}
	local className, classFilename, classId = UnitClass("player")
	local currentSpec = GetSpecialization()
	local id, name, description, icon, background, role = GetSpecializationInfo(currentSpec)
	MaxDps.SpellTable = MaxDpsSpellTable[idtoclass[classId]][idtospec[id]]
	--MaxDps.SpellInfoTable = {}
end

function MaxDps:UNIT_ENTERED_VEHICLE(_, unit)
	if unit == 'player' and self.rotationEnabled then
		self:DisableRotation();
	end
end

function MaxDps:UNIT_EXITED_VEHICLE(_, unit)
	if unit == 'player' then
		self:InitRotations();
		self:EnableRotation();
	end
end

function MaxDps:PLAYER_TARGET_CHANGED()
	if self.rotationEnabled then
		if UnitIsFriend('player', 'target') then
			return
		else
			self:InvokeNextSpell();
		end
	end
end

function MaxDps:PLAYER_REGEN_DISABLED()
	if self.db.global.onCombatEnter and not self.rotationEnabled then
		self:Print(self.Colors.Success .. 'Auto enable on combat!', "info");
		self:InitRotations();
		self:EnableRotation();
	end
end

function MaxDps:PLAYER_REGEN_ENABLED()
	self:DisableRotation();
end

function MaxDps:ButtonFetch()
	if self.rotationEnabled then
		if self.fetchTimer then
			self:CancelTimer(self.fetchTimer);
		end
		self.fetchTimer = self:ScheduleTimer('Fetch', 0.5);
	end
end

function MaxDps:PrepareFrameData()
	if not self.FrameData then
		self.FrameData = {
			cooldown  = self.PlayerCooldowns,
			activeDot = self.ActiveDots
		};
	end

	self.FrameData.timeShift, self.FrameData.currentSpell, self.FrameData.gcdRemains = MaxDps:EndCast();
	self.FrameData.gcd = self:GlobalCooldown();
	self.FrameData.buff, self.FrameData.debuff = MaxDps:CollectAuras();
	self.FrameData.talents = self.PlayerTalents;
	self.FrameData.azerite = self.AzeriteTraits;
	self.FrameData.essences = self.AzeriteEssences;
	self.FrameData.covenant = self.CovenantInfo;
	self.FrameData.runeforge = self.LegendaryBonusIds;
	self.FrameData.spellHistory = self.spellHistory;
	self.FrameData.timeToDie = self:GetTimeToDie();
end

function MaxDps:InvokeNextSpell()
	-- invoke spell check
	local oldSkill = self.Spell;

	self:PrepareFrameData();

	self:GlowConsumables();

	-- Removed backward compatibility
	self.Spell = self:NextSpell();

	if (oldSkill ~= self.Spell or oldSkill == nil) and self.Spell ~= nil then
		self:GlowNextSpell(self.Spell);
		if WeakAuras then
			WeakAuras.ScanEvents('MAXDPS_SPELL_UPDATE', self.Spell);
		end
	end

	if self.Spell == nil and oldSkill ~= nil then
		self:GlowClear();
		if WeakAuras then
			WeakAuras.ScanEvents('MAXDPS_SPELL_UPDATE', nil);
		end
	end
end

function MaxDps:InitRotations()
	self:Print(self.Colors.Info .. 'Initializing rotations', "info");
	self:CountTier()

	local _, _, classId = UnitClass('player');
	local spec = GetSpecialization();
	self.ClassId = classId;
	self.Spec = spec;

	if not self.Custom then
		self.Custom = self:GetModule('Custom');
	end

	self.Custom:LoadCustomRotations();
	local customRotation = self.Custom:GetCustomRotation(classId, spec);

	if customRotation then
		self.NextSpell = customRotation.fn;

		self:Print(self.Colors.Success .. 'Loaded Custom Rotation: ' .. customRotation.name, "info");
	else
		self:LoadModule();
	end
end

function MaxDps:LoadModule()
	if self.Classes[self.ClassId] == nil then
		self:Print(self.Colors.Error .. 'Invalid player class, please contact author of addon.', "error");
		return
	end

	local className = self.Classes[self.ClassId];
	local module = 'MaxDps_' .. className;
	local _, _, _, _, reason = GetAddOnInfo(module);

	if IsAddOnLoaded(module) then
		self:EnableRotationModule(className);
		return
	end

	if reason == 'MISSING' or reason == 'DISABLED' then
		self:Print(self.Colors.Error .. 'Could not find class module ' .. module .. ', reason: ' .. reason, "error");
		self:Print(self.Colors.Error .. 'Make sure to install class module or create custom rotation', "error");
		self:Print(self.Colors.Error .. 'Missing addon: ' .. module, "error");
		return
	end

	LoadAddOn(module);

	self:InitTTD();
	self:EnableRotationModule(className);
end

function MaxDps:EnableRotationModule(className)
	local loaded = self:EnableModule(className);

	if not loaded then
		self:Print(self.Colors.Error .. 'Could not find load module ' .. className .. ', reason: OUTDATED', "error");
	else
		self:Print(self.Colors.Info .. 'Finished Loading class module', "info");
	end
end
